import os, shutil

"""
    文件及目录处理相关的基类和方法
"""
# 目录处理类
class Contraler_Dir:
    # 获取当前根目录路径
    @staticmethod
    def getCurOriPath():
        return os.path.abspath(os.path.dirname(__file__))

    # 判断目录是否存在，不存在则创建
    @staticmethod
    def checkACreateDir(dirPath):
        exist = os.path.exists(dirPath)
        if (not exist):
            os.makedirs(dirPath)
        else:
            pass
        pass

    # 清空指定目录下所有文件
    @staticmethod
    def clearDirFiles(dirPath):
        lis = os.listdir(dirPath)
        for i in lis:
            os.remove(dirPath + i)

    @staticmethod
    def copyFile(src, dst):
        '''
            复制文件
        '''
        shutil.copyfile(src, dst)

    @staticmethod
    def renameFile(imgSrc, imgDst):
        """图片重命名"""
        os.rename(imgSrc, imgDst)



import cv2
import numpy as np
from customFunction__.Cracker.base_cracker import Base_Cracker
from selenium.webdriver.common.action_chains import ActionChains
import time

"""
    滑块验证 基类
    集成各种通用验证方法：
        滑块验证等
    注意 使用cv2 路径不能有中文名
"""
class Base_Slidercrack(Base_Cracker):
    def __init__(self, driverPath):
        Base_Cracker.__init__(self, driverPath)

    # 滑块验证 输出水平移动的距离 计算坐标的偏移量
    # templatePath 滑块 , targetPath 背景模板
    @staticmethod
    def getMovement_sliderCheck_method1(templatePath, targetPath, elWidth):
        '''
            滑块验证 输出水平移动的距离 计算坐标的偏移量
            注意：这一个算法没问题，只是可能需要试几次才有一次正确的
        :param elWidth: 背景图在网页上显示的宽度，用于比例计算实际滑动距离
        :param templatePath: 滑块图片路径
        :param targetPath: 背景图片路径
        :return: 偏移量
        '''
        target_rgb = cv2.imdecode(np.fromfile(targetPath, dtype=np.uint8), -1)
        target_gray = cv2.cvtColor(target_rgb, cv2.COLOR_BGR2GRAY)
        template_rgb = cv2.imread(templatePath, 0)
        ori_width = target_rgb.shape[1]
        ori_height = target_rgb.shape[0]
        res = cv2.matchTemplate(target_gray, template_rgb, cv2.TM_CCOEFF_NORMED)
        value = cv2.minMaxLoc(res)
        # (因为下载的图片跟网页上的图片是有缩放的)所以要根据比例计算网页端需要移动的偏移量
        effectiveVal = value[2][0] * elWidth / ori_width
        print('effectiveVal:', effectiveVal)
        print('value2:', value[2][0])
        print('value3:', value[3][0])
        print('res:', res.shape)
        return effectiveVal

    @classmethod
    def handle_SlideCheck(cls, *browser):
        '''处理滑块验证 根据不同站点的滑块验证进行重写'''
        pass

    @classmethod
    def download_slideImgs(cls, *browser, dstDirPath='E:\Projects\packageDIY\\videoRef\\assets\\'):
        '''
            下载滑块以及滑块背景用于识别位置 根据不同站点的滑块验证进行重写
        :return: targetWidth 背景图在网页中的宽度 用于步长计算
        '''
        pass

    # 模拟人操作
    def get_tracks(self, distance, rate=0.6, t=0.2, v=0):
        """
        将distance分割成小段的距离
        :param distance: 总距离
        :param rate: 加速减速的临界比例
        :param a1: 加速度
        :param a2: 减速度
        :param t: 单位时间
        :param t: 初始速度
        :return: 小段的距离集合
        """
        tracks = []
        # 加速减速的临界值
        mid = rate * distance
        # 当前位移
        s = 0
        # 循环
        while s < distance:
            # 初始速度
            v0 = v
            if s < mid:
                a = 20
            else:
                a = -3
            # 计算当前t时间段走的距离
            s0 = v0 * t + 0.5 * a * t * t
            # 计算当前速度
            v = v0 + a * t
            # 四舍五入距离，因为像素没有小数
            tracks.append(round(s0))
            # 计算当前距离
            s += s0
        return tracks


    def move_sliderTemplate(self, movement, *browser, templateSelector):
        '''
            移动滑块，根据不同站点进行重写
        :param movement:
        :return:
        '''
        if(browser==()):
            # 无指定浏览器
            # 实例化鼠标操作
            action = ActionChains(self.browser)
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            for i in range(movement):
                action.move_by_offset(1, 0)
            # 释放滑块
            action.release().perform()
        else:
            # 实例化鼠标操作
            action = ActionChains(browser[0])
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            for i in range(movement):
                action.move_by_offset(1, 0)
            # 释放滑块
            action.release().perform()

    def move_sliderTemplate_likeHuman(self, movement, *browser, templateSelector):
        '''
            移动滑块 根据不同站点进行重写
        :param movement: 移动的距离
        :param templateSelector: 滑块selector
        :return:
        '''
        if (browser == ()):
            # 无指定浏览器
            # 实例化鼠标操作
            action = ActionChains(self.browser)
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            _tracks = self.get_tracks(movement)
            new_1 = _tracks[-1] - (sum(_tracks) - movement)
            _tracks.pop()
            _tracks.append(new_1)
            print(_tracks)
            for mouse_x in _tracks:
                action.move_by_offset(mouse_x, 0)
            # 释放滑块
            action.release().perform()
            time.sleep(2)

        else:
            # 实例化鼠标操作
            action = ActionChains(browser[0])
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            _tracks = self.get_tracks(movement)
            new_1 = _tracks[-1] - (sum(_tracks) - movement)
            _tracks.pop()
            _tracks.append(new_1)
            print(_tracks)
            for mouse_x in _tracks:
                action.move_by_offset(mouse_x, 0)
            # 释放滑块
            action.release().perform()
            time.sleep(2)

    def get_selector(self):
        '''获取对应selector 根据不同站点进行重写'''
        pass

    @classmethod
    def slidecrack_run(cls):
        '''运行滑块破解功能 根据不同站点进行重写'''
        pass


'''
    自动化引擎
        缩略图 清洗 筛选 并上传
'''
from .globalTools import globalTools
from .imagesRef.Poster import Poster
from .imagesRef.universalTools import tools
from .imagesRef.Classifier import Classifier
from .imagesRef.Filter import Filter
from .imagesRef.Processing import Processing
from .imagesRef.DatabaserOperator import databaseOperator as dbOp
import os

def run(proj_absPath, oriDomain, database, tableNameList):
    # 将目录下的图片图片传到接口
    # 获取当前目录所在绝对路径
    # proj_absPath = os.path.abspath(os.path.dirname(__file__))
    updateTime = tools.getCurDate()
    '''
    oriDomain = 'huxiu'
    '''
    setting = {
        # 爬取下来的图片的存放路径
        'imgsCrawledDir' : proj_absPath + '\\assets\imgsCrawled\\' + updateTime + '\\' + oriDomain + '\\',
        # 经过百度识别重命名后存放的目录路径
        'imgsReconizedDir': proj_absPath + '\\assets\imgsReconized\\' + updateTime + '\\' + oriDomain + '\\',
        # 初步处理过后的无水印的图片的目录
        'imgsDirDontHasWaterMask' : proj_absPath + '\\assets\imgsDontHasWaterMask\\' + updateTime + '\\' + oriDomain + '\\',
        # 初步处理过后有水印的图片的目录
        'imgsDirHasWaterMask' : proj_absPath + '\\assets\imgDirHasWaterMask\\' + updateTime + '\\' + oriDomain + '\\',
        # 处理完成的图片目录
        'imgsCleanedDir' : proj_absPath + '\\assets\imgsCleanedDir\\' + updateTime + '\\' + oriDomain + '\\',
        # 缩略图保存位置图片目录
        'imgsThumbnailDir' : proj_absPath + '\\assets\imgsThumbnailDir\\' + updateTime + '\\' + oriDomain + '\\',
    }

    # 判断配置里的目录是否存在，不存在则创建对应目录
    for item in setting.values():
        tools.checkACreateDir(item)

    # 从数据库获取图片链接 下载图片
    print("从数据库获取图片链接")
    dbOperator = dbOp.dbOperator(database)
    # 上传过的图片去重
    picList_posted_ = dbOperator.getAllDataFromDB("SELECT `origin_pic_path` FROM `postedurldatabase`.`tb_thumbnailimgs_posted`")
    picList_posted = []
    for item in picList_posted_:
        picList_posted.append(item[0])
    print(picList_posted)
    print("下面开始下载图片")
    for table in tableNameList:
        sql = "SELECT `id`,`origin_pic_path` FROM `" + table + "`;"
        imgUrlPathList = dbOperator.getAllDataFromDB(sql)
        print("数据表为 ", table, "获得图片链接数量： ", len(imgUrlPathList))
        for imgUrl in imgUrlPathList:
            if(imgUrl[1] in picList_posted):
                continue
            imgName = table + "_" + str(imgUrl[0])
            tools.downimg(urlpath=imgUrl[1], imgname=imgName, dstDirPath=setting["imgsCrawledDir"])
            insert_SQL = "INSERT INTO `postedurldatabase`.`tb_thumbnailimgs_posted` (`origin_pic_path`) VALUES (\'{}\');".format(imgUrl[1])
            dbOperator.insertData2DB(insert_SQL)

    print("图片下载完成")
    # 对爬取下来的图片进行处理 - 识别重命名、过滤、水印的识别及裁切 处理完后放在路径  ./assets/imgsCleanedDir 下
    # 2 重命名
    classifier = Classifier.imgsClassifier(crawledDirPath=setting['imgsCrawledDir'], savedDirPath=setting['imgsReconizedDir'])
    classifier.run()

    # 3 过滤 (关键词过滤、空文件过滤、水印识别及处理）
    print("下面开始过滤操作")
    filter = Filter.imgsFilter(
        imgsDontHasWaterMaskDir=setting['imgsDirDontHasWaterMask'],
        imgDirHasWaterMask=setting['imgsDirHasWaterMask'],
        imgCleanedStep1 = setting['imgsCleanedDir'],
        # dirOriPath=setting['imgsReconizedDir']
        dirOriPath = setting['imgsCrawledDir']
    )
    filter.run_()    # 这里再做一下优化
    print("过滤操作完成")

    # 4 将过滤后的图片整成缩略图
    print("下面将图片整成缩略图")
    thumbnailOriDir = setting['imgsReconizedDir']
    thumbnailDstDir = setting['imgsThumbnailDir']
    imgprocesser = Processing.Imgs2ThumbnailByDir(thumbnailOriDirPath=thumbnailOriDir, thumbnailDstDirPath=thumbnailDstDir)
    imgprocesser.cutOff2ThumbnailByDir()

    print("下面将开始传送")
    # # 4 创建图片发送的poster 传送处理完成的图片
    # 传送缩略图
    imgposter0 = Poster.ImgPoster(imgDirPath=setting['imgsThumbnailDir'])
    imgposter0.updateImgsThumbnail()
    globalTools.finishTask()



from globalTools import globalTools
from basement__.ContralerTime import Contraler_Time
from basement__.ContralerDir import Contraler_Dir
from spider__ import selenium_douyin,selenium_sougou_weixin, selenium_kuaishou



# Selenium 爬取 爬取视频源 抖音
def run_douyin(proj_absPath, crawlUrl_list, oriDomain='douyin'):
    updateTime = Contraler_Time.getCurDate("%Y%m%d")
    videoDirPath = proj_absPath + '\\assets\\videos\\' + updateTime + '\\' + oriDomain + '\\'
    coverSavedPath = proj_absPath + '\\assets\\videos\\' + updateTime + '\\cover_douyin.jpg'
    captchaPath = proj_absPath + '\\assets\\captcha\\' + updateTime + '\\' + oriDomain + '\\'
    # 判断配置里的目录是否存在，不存在则创建对应目录
    Contraler_Dir.checkACreateDir(videoDirPath)
    Contraler_Dir.checkACreateDir(captchaPath)

    # 抖音视频的爬取及上传
    spider_douyin = selenium_douyin.Crawler_Douyin(captchaPath=captchaPath, videoDirPath=videoDirPath)
    for url in crawlUrl_list:
        lis = spider_douyin.enterIndexDouyin(move2BottomTimes=1, douyinUrlIndex=url)
        postResult = spider_douyin.getRealVideo(lis, videoDirPath, coverSavedPath)
    spider_douyin.browser0.close()
    spider_douyin.browser1.close()
    globalTools.finishTask()

def run_spider(origin):
    if(origin=='sougou'):
        sougou = selenium_sougou_weixin.Crawler_Sougou()
        sougou.run()
        globalTools.finishTask()
        del sougou
    elif (origin == 'kuaishou'):
        # 视频的爬取上传
        kuaishou = selenium_kuaishou.Crawler_Kuaishou()
        kuaishou.run()
        globalTools.finishTask()
        del kuaishou





from . import base_poster
import os, time, requests, hashlib
from requests_toolbelt import MultipartEncoder
from globalTools import globalTools
from basement__ import Tools as tools

'''
    Post 图片的类
        参数：
            imgDirPath  处理完的图片目录路径dir   注意这里的路径最后加\\的
'''
class Poster_Imgs(base_poster.Base_Poster):
    def __init__(self, imgDirPath):
        self.imgDirPath = imgDirPath
        self.get_imgPathList()

    # 获取目录下所有文件的路径列表
    def get_imgPathList(self):
        imgNameList = os.listdir(self.imgDirPath)  # 获取目录下的所有文件名
        imgPathList = []
        for imgName in imgNameList:
            item = self.imgDirPath + '\\' + imgName
            imgPathList.append(item)
        self.imgNameList = imgNameList
        return imgNameList

    # 获取时间戳
    def getCurTime(self):
        return int(round(time.time()))

    # 修改单张图片的md5
    def changeMD5(self, imgSrc):
        with open(imgSrc, 'rb') as f:
            md5 = hashlib.md5(f.read()).hexdigest()
        file = open(imgSrc, 'rb').read()
        with open(imgSrc, 'wb') as new_file:
            new_file.write(file + bytes('\0', encoding='utf-8'))  # here we are adding a null to change the file content
            newMD5 = hashlib.md5(open(imgSrc, 'rb').read()).hexdigest()
        # print("修改MD5的文件：", imgSrc, "\n旧MD5: ", md5, " \t 新MD5： ", newMD5)

    # post发送单张图片
    def post_imgSingle(self, imgName, interface='http://121.40.187.51:8088/api/contentimgs_api'):
        userName = 'qin'
        password = 'qin123456'
        curTime = self.getCurTime()
        imgPath = self.imgDirPath + "\\" + imgName
        print("处理的路径： ", imgPath)
        # 改md5
        self.changeMD5(imgPath)
        # 打开图片
        # f = open('E:\Projects\imgReconization\imgsDontHasWaterMask\\2+商品-家居家装-吸顶灯 + 非自然图像-图像素材-图像素材.jpg', 'rb')
        f = open(imgPath, 'rb')
        # fileByte = base64.b64encode(f.read())  # 不要用这个，不然图片显示不了
        curDate = tools.getCurDate()
        key = hashlib.md5(('datapool' + userName + password + str(curDate)).encode('utf-8')).hexdigest()
        # 这里传文件的时候用绝对路径传，不然传了之后显示不了
        formData = ({
                "key": key,
                "account": userName,
                "password": password,
                'file': (imgName, f, "image/jpeg"),
                'title': imgName.split('.')[0]
        })
        m = MultipartEncoder(formData)
        headers2 = {
                "Content-Type": m.content_type
        }
        imgPostResult = requests.post(url=interface, data=m, headers=headers2)
        f.close()

        try:
            # 上传完删除对应单张图片
            globalTools.delVideoSingle(imgPath)
        except Exception as e:
            print("删除图片出错： ", imgPath)
            print(e)

        return imgPostResult

    # 发送目录下的所有所有图(内容图)
    def updateImgs(self):
        for imgName in self.imgNameList:
            self.post_imgSingle(imgName)

    def updateImgsThumbnail(self):
        for imgName in self.imgNameList:
            self.post_imgSingle(imgName, interface='http://121.40.187.51:8088/api/thumbnail_api')






from selenium import webdriver
from time import sleep
import pymysql, time, requests, openpyxl, pyautogui
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from ..Poster.Poster import VideoPoster
from ..Filter.Filter import videoFilter
from ..universalTools import tools
from ..DatabaserOperator import databaseOperator as dbOp



# 已有链接下载的方法
def downVideo(urlpath, name, dstDirPath):
    # 获取当前日期
    r = requests.get(urlpath, verify=False)
    video = r.content       #响应的二进制文件
    with open(dstDirPath + str(name) + '.mp4','wb') as f:     #二进制写入
        f.write(video)
    r.close()   # 关闭很重要，确保不要过多的连接

"""
处理快手视频的封面图片src
"""
class crawlFromKuaishou():
    def __init__(self, url="https://www.kuaishou.com/search/video?searchKey=%E8%82%A1"):
        self.filter = videoFilter()
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_experimental_option('useAutomationExtension', False)
        self.browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe",options=option)
        self.browser.get(url)
        self.url = url
        # 登陆验证的
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "userId",
            "value": "2560728971",
            "path": '/',
            "expires": None
        })
        # 滑块验证的
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "did",
            "value": "web_dd4765fe13d667c461fd6269a10ae0df",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".www.kuaishou.com",
            "name": "kpf",
            "value": "PC_WEB",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".www.kuaishou.com",
            "name": "kpn",
            "value": "KUAISHOU_VISION",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "clientid",
            "value": "3",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "client_key",
            "value": "65890b29",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "kuaishou.server.web_ph",
            "value": "f026d92ae2b0265bf86ec666fa786f635d5b",
            "path": '/',
            "expires": None
        })
        self.browser.add_cookie({
            "domain": ".kuaishou.com",
            "name": "kuaishou.server.web_st",
            "value": "ChZrdWFpc2hvdS5zZXJ2ZXIud2ViLnN0EqABWHDql4qaYN0DtvJlDUI2vX9v2hDjf2eJUChEjJHM9YZiNRQRhDkcURY8NFHcPTyo3KaaN9tV-q9kFdDhaa6sLjoDSmhK5UjN2byF-eFVit8lfNtBhKzJ-dtvFTmsotI8eHddsdCisokwNMCBCXzdiwA7fmnM7SKqP3qEsXkJsOZJlJ1-CAUQSCJwVaVaU3SvYCeGb3Vi9BVfw0RtpgCzexoS8JByODRPv5hk-B95zTquvFHcIiCS6UA7jHPQUTMshtZGa63oO8ZPkqrI5c9ylvllsQswiCgFMAE",
            "path": '/',
            "expires": None
        })
        self.poster = VideoPoster(videoDirPath='E:\\test\\')
        print("请先登陆")

    # 获取clientCacheKey
    def getclientCacheKey(self, srcList):
        self.kuaishouSrcListOrigin = srcList
        clientCacheKeyList = []
        videoPageSrcList = []
        for src in self.kuaishouSrcListOrigin:
            if('clientCacheKey' in src):
                clientCacheKey = str(src).split("//")[1].split("clientCacheKey=")[1].split(".")[0]
                if(clientCacheKey not in clientCacheKeyList):
                    # 插入数据库操作
                    self.mySQLDataBaseInsertOp_clientCacheKey(clientCacheKey)
                    clientCacheKeyList.append(clientCacheKey)
            else:
                continue
        self.clientCacheKeyList = clientCacheKeyList
        for item in self.clientCacheKeyList:
            videoPageSrc = 'https://www.kuaishou.com/short-video/' + item
            videoPageSrcList.append(videoPageSrc)
        self.videoPageSrcList = videoPageSrcList
        return clientCacheKeyList

    def mySQLDataBaseInsertOp_clientCacheKey(self, clientCacheKey):
        # 设置数据库
        conn = pymysql.connect(
            host='localhost',
            user="root",
            passwd="root",
            db="articledatabase",
            autocommit=True
        )
        cursor = conn.cursor()
        sql = "INSERT INTO `articledatabase`.`tb_videopeiziclientcachekey` (`clientCacheKey`) VALUES (\'{}\');".format(
            clientCacheKey
        )
        # 执行Sql语句
        try:
            result = cursor.execute(sql)
            if (result == 1):
                # print("插入记录成功")
                pass
        except Exception as e:
            print(sql)
            print("插入记录失败： ", e)

    # 已经获取了clientCacheKey，下面这个方法已经不用了
    def crawlImgSrcBySelenium(self):
        # 1.创建Chrome浏览器对象，这会在电脑上在打开一个浏览器窗口
        # self.browser.get("https://www.kuaishou.com/search/video?searchKey=配资")
        # 2.通过浏览器向服务器发送URL请求
        # cookies = "kpf=PC_WEB; kpn=KUAISHOU_VISION; clientid=3; did=web_77c4eaa79392234fda0830a7abdcb2f7; client_key=65890b29"
        # cookieList = cookies.split(";")
        # for cookie in cookieList:
        #     self.browser.add_cookie({
        #         "domain": ".kuaishou.com",
        #         "name": cookie.split("=")[0],
        #         "value": cookie.split("=")[1],
        #         "path": '/',
        #         "expires": None
        #     })
        self.browser.get(self.url)

        sleep(3)
        element0 = self.browser.find_elements_by_xpath(
            "//div[@class='video-card-main']//div[@class='card-link']/div[@class='poster']/img[@class='poster-img']")
        print(len(element0))
        for item in element0:
            print(item.get_attribute('src'))

        # 下拉到底
        # js = "var q=document.documentElement.scrollTop=20000"
        self.browser.execute_script("""
            (function () {
            var y = 0;
            var step = 100;
            window.scroll(0, 0);
            function f() {
            if (y < document.body.scrollHeight) {
            y += step;
            window.scroll(0, y);
            setTimeout(f, 100);
            } else {
            window.scroll(0, 0);
                document.title += "scroll-done";
            }
            }
            setTimeout(f, 1000);
            })();
            """)
        print("下拉中...")
        time.sleep(2)
        # while (True):
        #     if ("scroll-done" in self.browser.title):
        #         break
        #     else:
        #         print("还没有拉到最底端...")
        #         sleep(3)
        n = 0
        while (n<=10):
            if ("scroll-done" in self.browser.title):
                break
            else:
                print("还没有拉到最底端...")
                sleep(3)
            n = n+1

        element1 = self.browser.find_elements_by_xpath("//div[@class='video-card-main']//div[@class='card-link']/div[@class='poster']/img[@class='poster-img']")
        clientCacheListOrigin = []
        for item in element1:
            clientCacheListOrigin.append(item.get_attribute('src'))
        # 处理爬取到的Src
        self.getclientCacheKey(clientCacheListOrigin)
        # 3.刷新浏览器
        self.browser.refresh()

    # 提取视频名字的方法
    def videoNameExtract(self, videoSrc):
        return str(videoSrc).split("?")[0].split("/")[-1]

    # 获取视频
    def getSingleVideoInfo(self, src):
        self.browser.get(src)
        sleep(3)
        # 获取视频信息
        publishTime = self.browser.find_element_by_xpath("//div[@class='short-video-info']//div[1]//div[@class='title-warp']//span[@class='photo-time']").text
        videoInfo = {}
        if(self.filter.checkIfCurDatePub_kuaishou(publishTime)):
            # 进入这里说明是当天发布的
            author = self.browser.find_element_by_xpath(
                "//div[@class='short-video-info']//div[1]//div[@class='title-warp']//span[@class='profile-user-name-title']").text
            videoTitle = self.browser.find_element_by_xpath("//div[@class='short-video-info-container-detail']/p").text
            videoSrc = self.browser.find_element_by_xpath("//video").get_attribute("src")
            videoName = self.videoNameExtract(videoSrc)
            videoInfo = {
                "author": author,
                "publishTime": publishTime,
                "videoTitle": videoTitle,
                "videoSrc": videoSrc,
                "videoName": videoName
            }
        return videoInfo

    def getVideoListInfo(self):
        videoInfoList = []
        for src in self.videoPageSrcList:
            videoInfo = self.getSingleVideoInfo(src)
            # 获取到视频信息， 开始下载并上传
            downVideo(urlpath=videoInfo['videoSrc'], name=videoInfo['videoName'], dstDirPath='E:\\test\\')
            videoInfoList.append(videoInfo)
            self.mySQLDataBaseInsertOp_videoInfo(videoInfo)
        self.videoInfoList = videoInfoList

    def getVideoListInfoByList(self,lis):
        for src in lis:
            videoInfo = self.getSingleVideoInfo(src)
            if(videoInfo):
                # 进入这里说明是视频是当天发布的，可以下载
                self.mySQLDataBaseInsertOp_videoInfo(videoInfo) # 视频信息放入数据库
                # 下载视频并上传
                downVideo(urlpath=videoInfo['videoSrc'], name=videoInfo['videoName'], dstDirPath='E:\\test\\')
                self.poster.post_videoSingle(videoInfo['videoName'] + '.mp4', title0=videoInfo["videoTitle"])


    def mySQLDataBaseInsertOp_videoInfo(self, videoInfoItem):
        # 设置数据库
        conn = pymysql.connect(
            host='localhost',
            user="root",
            passwd="root",
            db="articledatabase",
            autocommit=True
        )
        cursor = conn.cursor()
        sql = "INSERT INTO `articledatabase`.`tb_videopeizitest` (`title`, `author`, `time`, `videoSrc`, `videoName`) VALUES (\'{}\',\'{}\',\'{}\',\'{}\',\'{}\');".format(
                videoInfoItem["videoTitle"],
                videoInfoItem["author"],
                videoInfoItem["publishTime"],
                videoInfoItem["videoSrc"],
                videoInfoItem["videoName"]
            )
        # 执行Sql语句
        try:
            result = cursor.execute(sql)
            if (result == 1):
               # print("插入记录成功")
               pass
        except Exception as e:
            print(sql)
            print("插入记录失败： ", e)

    def run(self):
        self.crawlImgSrcBySelenium()
        # # 从数据库获取clientCacheKey
        # conn = pymysql.connect(
        #     host='localhost',
        #     user="root",
        #     passwd="root",
        #     db="articledatabase",
        #     autocommit=True
        # )
        # cursor = conn.cursor()
        # sql = "SELECT `clientCacheKey` FROM `articledatabase`.`tb_videopeiziclientcachekey`"
        # # 执行Sql语句
        # try:
        #     cursor.execute(sql)
        #     result = cursor.fetchall()
        # except Exception as e:
        #     pass
        # videoPageSrcList = []
        # for clientCacheKeyItem in result:
        #     videoPageSrc = 'https://www.kuaishou.com/short-video/' + clientCacheKeyItem[0]
        #     videoPageSrcList.append(videoPageSrc)
        self.getVideoListInfoByList(self.videoPageSrcList)



from selenium import webdriver
from time import sleep
import requests
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from customFunction__.Poster.poster_video import Poster_Video as VideoPoster
from customFunction__.Filter.filter_video import DouyinFilter
from basement__.ContralerTime import Contraler_Time
from basement__ import Tools as tools
from basement__ import ContralerDatabase as dbOp
from customFunction__.Cracker import cracker_douyin
from customFunction__.Cleaner.cleaner_title import Cleaner_Title

# 已有图片链接下载图片的方法
def downVideo(urlpath, name, dstDirPath):
    print("下载： ", urlpath)
    r = requests.get(urlpath, verify=False)
    video = r.content       #响应的二进制文件
    with open(dstDirPath + str(name) + '.mp4','wb') as f:     #二进制写入
        f.write(video)
    r.close()   # 关闭很重要，确保不要过多的连接

# --------------------------- 爬取抖音视频的类 ----------------------------------
"""
    获得视频列表
    过滤视频列表
        筛选条件：
        1 判断视频时长 1~6min
        2 判断发布时间为当天
        3 标题筛选 筛掉包含过滤词的视频
        上传视频
"""
class Douyin_SpecialMethod:
    @staticmethod
    def check_publishtime_istoday(s: str):
        """
        根据抖音视频页面的发布时间表示情况写的方法 判断是否为当天
        :param s 类似 2021-12-07 的字符串
        :output Boolean
        """
        pubTime = "".join(s.split("-"))
        if (pubTime != Contraler_Time.getCurDate("%Y%m%d")):
            # 判断不是当天的视频则跳过
            return False
        else:
            return True

    @staticmethod
    def check_timelength_between(s, min=1, max=6):
        """
        判断视频长度是否符合条件
        :param s 视频长度字符串 如 06:30
        :param min 最小几分钟
        :param max 最大几分钟
        根据抖音视频页面的发布时间表示情况写的方法 判断视频长度是否符合要求： >=1min, <=6min
        """
        if (s.split(":")[0].startswith('0')):
            if (s.split(":")[0].replace('0', '') != ''):
                minute = s.split(":")[0].lstrip('0')
            else:
                minute = '0'
            if (int(minute) >= min and int(minute) <= max):
                # 符合条件
                return True
            else:
                return False
        else:
            return False

    # 处理滑块验证的方法：1 直接关闭 2 滑动验证
    def handleSlideCheck(self):
        sleep(5)
        try:
            self.EffectiveCookies0 = self.douyinCracker0.handle_SlideCheck(self.browser0)
        except Exception as e:
            print("browser0 无滑块出现")

        try:
            self.EffectiveCookies1 = self.douyinCracker1.handle_SlideCheck(self.browser1)
        except Exception as e:
            print("browser1 无滑块出现")

    # 设置上一次爬取最新的那个数据的信息
    def setTheNewestCrawledInfo(self, title):
        self.theNewestTitle = title

    # 下滑到最底部
    def moveToBottom(self, times=350):
        # 以当前位置为参照向下滚动
        # self.browser.execute_script('window.scrollBy(0 ,1000)') # 从当前位置向下滚动1000px
        for i in range(0, times):
            self.browser0.find_element_by_tag_name('body').send_keys(Keys.ARROW_DOWN)  # 在这里使用模拟的下方向键
            sleep(0.01)

    # 滑块验证 XX 滑块验证有点问题， 手动验证吧，自动化滑块验证还做不了
    # 遇到的问题，滑块验证采用拼图的形式，需要先检验待拼的位置坐标才行
    #   抖音的验证流程， 服务器发送一个cookie s_v_web_id:Value 到客户端，客户端进行滑块验证，验证成功的话发送一个成功的请求到服务器端，服务器端对 对应s_v_web_id  做一些调整（赋予权限），客户端就能正常浏览了
    #   s_v_web_id在一次会话结束时候失效
    def checkSlide(self, browser, x):
        sleep(2)
        # 实例化鼠标操作
        action = ActionChains(browser)
        # 按住滑块
        action.click_and_hold(browser.find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")).perform()
        for i in range(x):
            action.move_by_offset(1, 0)
        sleep(2)
        # 释放滑块
        action.release().perform()
        pass


class Crawler_Douyin(Douyin_SpecialMethod):
    def __init__(self, captchaPath, videoDirPath, chromeDriverPath=r'E:\Projects\webDriver\\chrome\\chromedriver.exe'):
        self.dboperator = dbOp.Contraler_Database(databaseName='postedurldatabase')
        self.filter = DouyinFilter(dirOriPath=videoDirPath)

        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_argument('--disable-blink-features=AutomationControlled')

        self.browser0 = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        self.browser1 = webdriver.Chrome(executable_path=chromeDriverPath, options=option)
        self.browser0.get('https://www.douyin.com')
        self.browser1.get('https://www.douyin.com')
        sleep(2)
        self.douyinCracker0 = cracker_douyin.DouyinCrack(captchaDstDirPath=captchaPath)
        self.douyinCracker1 = cracker_douyin.DouyinCrack(captchaDstDirPath=captchaPath)
        sleep(2)

        self.handleSlideCheck() # 滑块验证
        self.theNewestTitle = ''

        # 滑块验证参数
        self.captchaPath = captchaPath

    # 首次进入 move2BottomTimes 为向下滑动的次数 默认350
    def enterIndexDouyin(self, move2BottomTimes, douyinUrlIndex='https://www.douyin.com/search/%E8%82%A1%E7%A5%A8?publish_time=1&sort_type=2&source=normal_search&type=video'):
        self.browser0.execute_cdp_cmd('Page.addScriptToEvaluateOnNewDocument', {
            'source': 'Object.defneProperty(navigator, "webdriver", {get: () => undefined})'
        })
        self.browser0.get(douyinUrlIndex)
        # 滑块验证
        self.handleSlideCheck()

        # 2.通过浏览器向服务器发送URL请求
        self.browser0.get(douyinUrlIndex)
        # 滑块验证
        self.handleSlideCheck()

        # 等待某个元素是否出现
        WebDriverWait(self.browser0, 10).until(
            # EC.text_to_be_present_in_element((By.XPATH, ''))
            EC.presence_of_element_located((By.XPATH, "//ul[@class='_3636d166d0756b63d5645bcd4b9bcac4-scss']"))
        )
        self.handleSlideCheck()
        # 这里获取的ul用于判断
        try:
            ul = self.browser0.find_element_by_xpath("//ul[@class='_3636d166d0756b63d5645bcd4b9bcac4-scss']")
            liList = ul.find_elements_by_xpath("./li")
        except Exception as e:
            # 滑块验证
            self.handleSlideCheck()


        try:
            liFirstTitle = liList[0].find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']") # 获取标题
        except Exception as e:
            liFirstTitle = '空'
            # 滑块验证
            self.handleSlideCheck()

        liEffectiveList = []  # 可上传的视频信息列表
        # 判断是否有上一次爬取
        if(self.theNewestTitle!=''):
            # 说明有上一次爬取
            self.moveToBottom(move2BottomTimes)
            if(not(liFirstTitle == self.theNewestTitle)):
                # 说明有更新
                for li in liList:
                    # 每获取一个则下载上传一个
                    try:
                        a = li.find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                    except Exception as e:
                        # 滑块验证
                        self.handleSlideCheck()
                        continue
                    try:
                        publishTime = li.find_element_by_xpath(".//span[@class='b32855717201aaabd3d83c162315ff0a-scss']").text
                        timeLength = li.find_element_by_xpath(".//span[@class='d170ababc38fdbf760ca677dbaa9206a-scss']")
                    except Exception as e:
                        # 滑块验证
                        self.handleSlideCheck()
                    title = a.text
                    videoPageUrl = a.get_attribute("href")

                    # 当爬取到最新的那个则跳出循环
                    if(title == self.theNewestTitle):
                        break
                    if (self.check_timelength_between(timeLength)):
                        liEffectiveList.append((title, videoPageUrl, publishTime))
                    else:
                        continue
                # 更新最新的那个数据
                self.setTheNewestCrawledInfo(
                    liList[0].find_element_by_xpath(
                        ".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                )
            return liEffectiveList
        else:
            # 滑块验证
            self.handleSlideCheck()
            # 向下滚动
            self.moveToBottom(move2BottomTimes)

            ul = self.browser0.find_element_by_xpath("//ul[@class='_3636d166d0756b63d5645bcd4b9bcac4-scss']")
            liList = ul.find_elements_by_xpath("./li")
            self.liList = liList
            for li in liList:
                # 每获取一个则下载上传一个
                try:
                    a = li.find_element_by_xpath(".//a[@class='caa4fd3df2607e91340989a2e41628d8-scss a074d7a61356015feb31633ad4c45f49-scss _9c976841beef15a22bcd1540d1e84c02-scss']")
                except Exception as e:
                    continue
                timeLength = li.find_element_by_xpath(".//span[@class='d170ababc38fdbf760ca677dbaa9206a-scss']").text
                publishTime = li.find_element_by_xpath(".//span[@class='b32855717201aaabd3d83c162315ff0a-scss']").text
                title = a.text
                videoPageUrl = a.get_attribute("href")
                if (self.check_timelength_between(timeLength)):
                    liEffectiveList.append((title, videoPageUrl, publishTime))
                else:
                    continue
                sleep(5)
            return liEffectiveList




    def getRealVideo(self, videoList_, videoDirPath, coverSavedPath):
        poster = VideoPoster(videoDirPath=videoDirPath, coverSavedPath=coverSavedPath)
        filter_video = DouyinFilter(dirOriPath=videoDirPath)
        if (videoList_):
            videoList = self.filter.filter_posted(videoList_) # 过滤掉上传过的视频
            videoList = tools.cleanRepeated(videoList)  # 去重
            # 过滤标题操作
            videoList = filter_video.filter_keywordFromTitle(videoList)
        else:
            return None
        i = 1
        if(videoList!=''):
            for item in videoList:
                # 2.通过浏览器向服务器发送URL请求
                self.browser1.get(item[1])
                # 滑块验证
                self.handleSlideCheck()
                self.browser1.get(item[1])
                sleep(5)
                # 说明出现了滑块验证处理一下
                self.handleSlideCheck()

                # 获取发布时间，判断发布时间是否是当天，是的话才进行下一步操作，不是的话跳出循环进入下一个循环
                pubTime = self.browser1.find_element_by_xpath("//span[@class='_87bab22a14dd86d6a0038ee4b3fdaea4-scss']").text.split("：")[1]
                if(not self.check_publishtime_istoday(pubTime)):
                    # 判断不是当天的视频则跳过
                    continue
                # 获取准确的可下载的视频链接
                try:
                    videoUrl = self.browser1.find_element_by_xpath("//video//source")
                    if (len(videoUrl) > 1):
                        videoUrl = videoUrl[0].get_attribute("src")
                    else:
                        videoUrl = videoUrl.get_attribute("src")
                except Exception as e:
                    continue


                # 抖音标题删除标签
                title = Cleaner_Title.clean_douyin_method2(title=item[0])
                if(title.replace(' ','')==''):
                    continue

                downVideo(urlpath=videoUrl, name=str(i), dstDirPath=videoDirPath)
                # 上传
                print("上传视频: ", i)
                poster.post_videoSingle(str(i) + '.mp4', title0=title)

                # 更新上传过的数据库 postedurldatabase
                sql = "INSERT INTO `postedurldatabase`.`tb_video_posted` (`title`) VALUES ('{}');".format(
                    item[0]
                )
                self.dboperator.insertData2DB(sql=sql)
                i = i + 1
            return videoList
        else:
            return '无更新'





import re, os
import datetime

from selenium import webdriver
import pymysql
from selenium.webdriver.common.action_chains import ActionChains
import random, time
import requests
import shutil
# from .. import globalTools

'''
    返回毫秒级时间戳
'''
def getMilliSecond():
    return int(round(time.time() * 1000))

class cleanCookiesAndHeadersByRow():
    def translateCookies(self, row):
        cookieList = row.split(";")
        cookies = {}
        for item in cookieList:
            key = item.split("=")[0]
            val = item.split("=")[1]
            cookies[str(key).strip()] = str(val).strip()
        return cookies
    def translateHeaders(self, row):
        headerList = row.split("\n")
        headers = {}
        for item in headerList:
            key = item.split(":")[0].strip()
            if(key == 'Referer'):
                val = item[9:]
            else:
                try:
                    val = str(item).split(":")[1]
                except Exception as e:
                    print(e)

            if(key!=''):
                headers[key] = str(val).strip()
        return headers

"""
:param  articleItem  Pipeline的Item
"""
class cleanArticleItem():
    def __init__(self, artivleItem):
        self.articleItem = artivleItem
        self.setCleanedArticleItem()

    def setCleanedArticleItem(self):
        temp = self.cleanArticleItem()
        self.cleaned_articleItem = temp
        return None

    def getCleanedArticleItem(self):
        return self.cleaned_articleItem

    def cleanFuncBasic(self, s):
        if(re.search(':',str(s)) or re.search('：',str(s))):
            sList = str(s).split("：") or str(s).split(":")
            result = "".join(sList[1].strip())
            return result
        else:
            print("传入的字符串没有冒号，不能分离")
            return None
    def translateTime(self, s):
        return datetime.datetime.strptime(str(s), "%Y-%m-%d %H:%M:%S")

    def cleanContent(self, lis):
        for item in lis:
            item = item.replace('\u3000','')
        strContent = "".join(lis).replace('\'','\"')
        return strContent
    # 返回图片名，无.jpg
    def cleanImgSrc2Name(self, s):
        return str(str(s).split('/')[-1]).split('.')[0]

    def cleanArticleItem(self):
        temp = {}
        temp['articleTitle'] = self.articleItem['articleTitle']
        temp['articleAbstract'] = self.articleItem['articleAbstract']
        temp['articleFrom'] = self.cleanFuncBasic(self.articleItem['articleFrom'])
        temp['articleAuthor'] = self.cleanFuncBasic(self.articleItem['articleAuthor'])
        temp['articleTime'] = self.translateTime(self.cleanFuncBasic(self.articleItem['articleTime']))
        temp['articleContent'] = self.cleanContent(self.articleItem['articleContent'])
        temp['articleImgSrc'] = self.articleItem['articleImgSrc']
        # 从图片路径提取图片名
        temp['articleImgName'] = self.cleanImgSrc2Name(self.articleItem['articleImgSrc'])
        return temp


# --------------------------- 爬取今日头条财经目录下所有图片的类 ----------------------------------
'''
    使用框架 selenium
    没有有url，获取文章的url
'''
class articleUrl_toutiao():
    # 代理服务器(产品官网 www.16yun.cn)
    proxyHost = "u7125.5.tp.16yun.cn"
    proxyPort = "6445"

    # 代理验证信息
    proxyUser = "16HMCXON"
    proxyPass = "387861"

    # def create_proxy_auth_extension(proxy_host, proxy_port,
    #                                 proxy_username, proxy_password,
    #                                 scheme='http', plugin_path=None):
    #
    #     # 代理服务器(产品官网 www.16yun.cn)
    #     proxyHost = "u7125.5.tp.16yun.cn"
    #     proxyPort = "6445"
    #
    #     # 代理验证信息
    #     proxyUser = "16HMCXON"
    #     proxyPass = "387861"
    #
    #     proxy_host = proxyHost
    #     proxy_port = proxyPort
    #     proxy_username = proxyUser
    #     proxy_password = proxyPass
    #
    #     if plugin_path is None:
    #         plugin_path = r'D:/{}_{}@t.16yun.zip'.format(proxy_username, proxy_password)
    #
    #     manifest_json = """
    #     {
    #         "version": "1.0.0",
    #         "manifest_version": 2,
    #         "name": "16YUN Proxy",
    #         "permissions": [
    #             "proxy",
    #             "tabs",
    #             "unlimitedStorage",
    #             "storage",
    #             "",
    #             "webRequest",
    #             "webRequestBlocking"
    #         ],
    #         "background": {
    #             "scripts": ["background.js"]
    #         },
    #         "minimum_chrome_version":"22.0.0"
    #     }
    #     """
    #
    #     background_js = string.Template(
    #         """
    #         var config = {
    #             mode: "fixed_servers",
    #             rules: {
    #                 singleProxy: {
    #                     scheme: "${scheme}",
    #                     host: "${host}",
    #                     port: parseInt(${port})
    #                 },
    #                 bypassList: ["foobar.com"]
    #             }
    #           };
    #
    #         chrome.proxy.settings.set({value: config, scope: "regular"}, function() {});
    #
    #         function callbackFn(details) {
    #             return {
    #                 authCredentials: {
    #                     username: "${username}",
    #                     password: "${password}"
    #                 }
    #             };
    #         }
    #
    #         chrome.webRequest.onAuthRequired.addListener(
    #             callbackFn,
    #             {urls: [""]},
    #             ['blocking']
    #         );
    #         """
    #     ).substitute(
    #         host=proxy_host,
    #         port=proxy_port,
    #         username=proxy_username,
    #         password=proxy_password,
    #         scheme=scheme,
    #     )
    #
    #     with zipfile.ZipFile(plugin_path, 'w') as zp:
    #         zp.writestr("manifest.json", manifest_json)
    #         zp.writestr("background.js", background_js)
    #
    #     return plugin_path
    #
    # proxy_auth_plugin_path = create_proxy_auth_extension(
    #     proxy_host=proxyHost,
    #     proxy_port=proxyPort,
    #     proxy_username=proxyUser,
    #     proxy_password=proxyPass)
    #
    # option = webdriver.ChromeOptions()
    #
    # option.add_argument("--start-maximized")
    #
    # # 如报错 chrome-extensions
    # # option.add_argument("--disable-extensions")
    #
    # option.add_extension(proxy_auth_plugin_path)
    #
    # # 关闭webdriver的一些标志
    # # option.add_experimental_option('excludeSwitches', ['enable-automation'])
    #
    # driver = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", chrome_options=option)
    #
    # # 修改webdriver get属性
    # # script = '''
    # # Object.defineProperty(navigator, 'webdriver', {
    # # get: () => undefined
    # # })
    # # '''
    # # driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {"source": script})
    # result = driver.get("http://httpbin.org/ip")
    # print(result)
    # pass

    def get_cookiesAndSignature(self):
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_experimental_option('useAutomationExtension', False)
        browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", options=option)
        start_urls = "https://www.toutiao.com/"
        # 2.通过浏览器向服务器发送URL请求
        # browser.get("http://httpbin.org/ip")    # 查看本机ip，看代理是否起作用

        browser.get(start_urls)
        time.sleep(1)  # 等待
        # xhr重写
        js = """(function() {
                var proxied = window.XMLHttpRequest.prototype.open;
                let root = document.getElementById("root");
                let urlLinkNode = document.createElement("div");
                urlLinkNode.setAttribute("id", "urlLinkNode");
                root.appendChild(urlLinkNode);
                window.XMLHttpRequest.prototype.open = function() {
                    if(arguments[0]=='GET' && arguments[1].search("channel_id=3189399007")){
                        console.log(arguments);
                        let urlLinkNodeReal = document.getElementById("urlLinkNode")
                        urlLinkNodeReal.innerHTML = arguments[1];
                        return proxied.apply(this, [].slice.call(arguments));
                    }else{
                        return 0
                    }
                };
            })();"""
        browser.execute_script(js)

        # 定位
        caijing = browser.find_element_by_xpath("//div[@class='left-container']//ul[@class='feed-default-nav']/li[4]")
        time.sleep(2)
        caijing.click()

        # 获取最新signature
        signatureApi = browser.find_element_by_id("urlLinkNode").text
        signature = str(signatureApi).split("signature=")[1]
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        browser.close()
        return {
            "cookies":cookies,
            "signature":signature
        }

    def open(self):
        pass

'''
    使用框架 selenium
    已有url，下载url下的图片
'''
class ImgsDownloadByUrl():
    def __init__(self):
        self.conn = pymysql.connect(
            host='localhost',
            user="root",
            passwd="root",
            db="articledatabase",
            autocommit=True
        )
        self.cursor = self.conn.cursor()
        self.gerUrlFromMysql()

    def gerUrlFromMysql(self):
        sql = "SELECT * FROM `articledatabase`.`tb_imgstoutiaocaijing`;"
        self.cursor.execute(sql)
        urlList = self.cursor.fetchall()
        self.urlList = urlList
        self.closeMysql()
        return urlList

    # def download(self):
    #     option = webdriver.ChromeOptions()
    #     option.add_experimental_option('excludeSwitches', ['enable-automation'])
    #     option.add_experimental_option('useAutomationExtension', False)
    #     browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", options=option)
    #     self.urlList = self.urlList[5781:]
    #     # 2.通过浏览器向服务器发送URL请求
    #     for item in self.urlList:
    #         browser.get(item[1])
    #         time.sleep(0.5)  # 等待
    #         # 定位
    #         img = browser.find_element_by_xpath("//img")
    #         actions = ActionChains(browser)
    #         # 找到图片后右键单击图片
    #         actions.move_to_element(img)  # 定位到元素
    #         pyautogui.hotkey("ctrl", "s")
    #         time.sleep(0.5)  # 等待一秒
    #         # 输入视频名字
    #         pyautogui.typewrite(str(item[0]) + '.jpg')
    #         pyautogui.hotkey('enter')
    #         sleep(1)
    #     browser.close()

    def file_path(self, request, response=None, info=None, *, item=None):
        # 提取图片名
        imgName = str(str(request.url).split('/')[-1]).split('?')[0]
        return f'toutiaoImgs/{imgName}.jpg'

    def closeMysql(self):
        self.cursor.close()
        self.conn.close()


# --------------------------- 随机时间休眠 ----------------------------------
# 随机时间
def randomSleep():
    sleepTime = random.randint(0,50)
    time.sleep(sleepTime)
    pass

# --------------------------- 东方财富 cookie 和 signature 自动获取 -----------
class ArticleTool_DongFangCaiFu():
    def get_cookies_auto(self):
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_experimental_option('useAutomationExtension', False)
        browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", options=option)
        start_urls = "https://www.eastmoney.com/"
        # 2.通过浏览器向服务器发送URL请求
        # browser.get("http://httpbin.org/ip")    # 查看本机ip，看代理是否起作用
        browser.get(start_urls)
        time.sleep(1)  # 等待
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        browser.close()
        return cookies

# 已有图片链接下载图片的方法
def downimg(urlpath, imgname, dstDirPath):
    # 获取当前日期
    r = requests.get(urlpath)
    img = r.content       #响应的二进制文件
    with open(dstDirPath + str(imgname) + '.png','wb') as f:     #二进制写入
        f.write(img)


# 图片重命名
def reName(imgSrc, imgDst):
    os.rename(imgSrc, imgDst)


# 获取accessToken的方法
def getAccessToken(AK, SK):
    # client_id 为官网获取的AK， client_secret 为官网获取的SK
    host = 'https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id={}&client_secret={}'.format(AK, SK)
    response = requests.get(host)
    if response:
        return response.json()['access_token']


# 复制文件
def copyFile(src, dst):
    shutil.copyfile(src, dst)


def checkIfAllNumber(str):
    # 验证字符串是否全部为数字
    p = re.compile('^[0-9]*$')
    result = p.match(str)
    if(result):
        # 说明全部位数字，返回1
        return 1
    else:
        return 0

# 判断目录是否存在，不存在则创建
def checkACreateDir(dirPath):
    exist = os.path.exists(dirPath)
    if(not exist):
        os.makedirs(dirPath)
    else:
        pass
    pass

# 获取当前日期
def getCurDate():
    return time.strftime("%Y%m%d", time.localtime())

# 获取当前图片下载的目录路径 oriDomain为带爬取的网站的名字，方便分类
def getImsgDownloadDirPath(oriDomain='toutiao'):
    proj_absPath = globalTools.getCurOriPath()
    updateTime = getCurDate()
    return proj_absPath + '\\assets\imgsCrawled\\' + updateTime + '\\' + oriDomain + '\\'



import time

from .globalTools import GetParams_Selenium
from .globalTools import Downloader
from .sliderCheck import SliderChecker
from selenium.webdriver.common.action_chains import ActionChains


class DouyinCrack(GetParams_Selenium):
    '''
        可以根据参数driver的不同针对不同driver做的验证码破解
        但这里的方案是通过破解滑块验证来获取有效cookies，之后再通过接口获取数据
    '''
    def __init__(self, *driverPath, captchaDstDirPath):
        if(driverPath!=()):
            # 显示继承父类
            GetParams_Selenium.__init__(self, driverPath)
            self.downloader = Downloader()  # 资源下载器
            self.captchaDstDirPath = captchaDstDirPath
        else:
            self.downloader = Downloader()  # 资源下载器
            self.captchaDstDirPath = captchaDstDirPath


    '''
        抖音的滑块验证以及获取所需要的参数
    '''
    def enter_url(self, url):
        '''
        进入指定链接
        :param url: 注意需要加上协议 如：https://
        :return:
        '''
        self.browser.get(url)
        pass

    def del_All_cookies(self):
        '''
        删除本地保存的所有cookie，以便网站弹出验证框
        :return: none
        '''
        self.browser.delete_all_cookies()
        pass

    def handle_SlideCheck(self, *browser):
        '''
            处理指定browser的抖音滑块验证
        :param browser: 指定的browser
        :return:
        '''
        while (1):
            if(browser==()):
                # 无指定浏览器
                targetWidth = self.download_img_sliderCheck(dstDirPath=self.captchaDstDirPath)  # 滑块图片下载器 返回目标网页显示的宽
                movement = SliderChecker.getMovement_sliderCheck_douyin_static(self.captchaDstDirPath+'template.png', self.captchaDstDirPath+'target.png', targetWidth)  # 验证码处理器
                print("滑动的实际距离：", movement)
                # 滑动滑块
                # self.move_sliderTemplate(int(movement))
                self.move_sliderTemplate_likeHuman(int(movement))
            else:
                targetWidth = self.download_img_sliderCheck(browser[0],dstDirPath=self.captchaDstDirPath)  # 滑块图片下载器 返回目标网页显示的宽
                movement = SliderChecker.getMovement_sliderCheck_douyin_static(self.captchaDstDirPath+'template.png', self.captchaDstDirPath+'target.png', targetWidth)
                print("滑动的实际距离：", movement)
                # 滑动滑块
                # self.move_sliderTemplate(int(movement))
                self.move_sliderTemplate_likeHuman(int(movement), browser[0])

            # 滑完一会查看是否需要验证，不需要则滑块验证通过，否则重新执行该方法
            time.sleep(2)
            try:
                if(browser==()):
                    # 无指定浏览器
                    sliderTargetSelector = self.browser.find_element_by_xpath("//img[@id='captcha-verify-image']")  # 背景
                else:
                    sliderTargetSelector = browser.find_element_by_xpath("//img[@id='captcha-verify-image']")  # 背景

            except Exception as e:
                print("验证通过")
                break
            # 验证通过后输出cookie
        return self.get_Obj_allCookies()

    def download_img_sliderCheck(self, *browser, dstDirPath='E:\Projects\packageDIY\\videoRef\\assets\\'):
        '''
        下载滑块以及滑块背景用于识别位置
        :return: targetWidth 背景图在网页中的宽度 用于步长计算
        '''
        try:
            # 背景 target
            # 滑块 template
            if(browser==()):
                    # 无指定浏览器
                sliderTargetSelector = self.browser.find_element_by_xpath("//img[@id='captcha-verify-image']")  # 背景
                sliderTemplateSelector = self.browser.find_element_by_xpath("//img[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']") # 滑块
            else:
                sliderTargetSelector = browser[0].find_element_by_xpath("//img[@id='captcha-verify-image']")  # 背景
                sliderTemplateSelector = browser[0].find_element_by_xpath("//img[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")  # 滑块
            print("出现滑块，下载识别滑块")

            targetImgUrl = sliderTargetSelector.get_attribute('src')    # 背景图url
            templateImgUrl = sliderTemplateSelector.get_attribute('src')    # 滑块url
            targetWidth = sliderTargetSelector.size['width']     # 背景图片宽
            templateWidth = sliderTemplateSelector.size['width']    # 滑块宽
            # 下载模板图片——滑块
            self.downloader.download_img(templateImgUrl, 'template', dstDirPath)
            # 下载背景图片
            self.downloader.download_img(targetImgUrl, 'target', dstDirPath)
            return targetWidth
        except Exception as e:
            # print("无滑块出现，下载失败请刷新浏览器重新操作")
            return ''

    def move_sliderTemplate(self, movement, *browser):
        '''
            移动滑块，移动速度慢了点，后面再想办法处理一下
        :param movement:
        :return:
        '''
        if(browser==()):
            # 无指定浏览器
            # 实例化鼠标操作
            action = ActionChains(self.browser)
            templateSelector = self.browser.find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']") #滑块
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            for i in range(movement):
                action.move_by_offset(1, 0)
            # 释放滑块
            action.release().perform()
        else:
            # 实例化鼠标操作
            action = ActionChains(browser[0])
            templateSelector = browser[0].find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")  # 滑块
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            for i in range(movement):
                action.move_by_offset(1, 0)
            # 释放滑块
            action.release().perform()


    # 模拟人操作
    def get_tracks(self, distance, rate=0.6, t=0.2, v=0):
        """
        将distance分割成小段的距离
        :param distance: 总距离
        :param rate: 加速减速的临界比例
        :param a1: 加速度
        :param a2: 减速度
        :param t: 单位时间
        :param t: 初始速度
        :return: 小段的距离集合
        """
        tracks = []
        # 加速减速的临界值
        mid = rate * distance
        # 当前位移
        s = 0
        # 循环
        while s < distance:
            # 初始速度
            v0 = v
            if s < mid:
                a = 20
            else:
                a = -3
            # 计算当前t时间段走的距离
            s0 = v0 * t + 0.5 * a * t * t
            # 计算当前速度
            v = v0 + a * t
            # 四舍五入距离，因为像素没有小数
            tracks.append(round(s0))
            # 计算当前距离
            s += s0
        return tracks

    def move_sliderTemplate_likeHuman(self, movement, *browser):
        '''
            移动滑块，移动速度慢了点，后面再想办法处理一下
        :param movement:
        :return:
        '''
        if (browser == ()):
            # 无指定浏览器
            # 实例化鼠标操作
            action = ActionChains(self.browser)
            templateSelector = self.browser.find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")  # 滑块
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            _tracks = self.get_tracks(movement)
            new_1 = _tracks[-1] - (sum(_tracks) - movement)
            _tracks.pop()
            _tracks.append(new_1)
            print(_tracks)
            for mouse_x in _tracks:
                action.move_by_offset(mouse_x, 0)
            # 释放滑块
            action.release().perform()
            time.sleep(2)

        else:
            # 实例化鼠标操作
            action = ActionChains(browser[0])
            templateSelector = browser[0].find_element_by_xpath("//*[@class='captcha_verify_img_slide react-draggable sc-VigVT ggNWOG']")  # 滑块
            # 按住滑块
            action.click_and_hold(templateSelector).perform()

            print("验证中...")
            _tracks = self.get_tracks(movement)
            new_1 = _tracks[-1] - (sum(_tracks) - movement)
            _tracks.pop()
            _tracks.append(new_1)
            print(_tracks)
            for mouse_x in _tracks:
                action.move_by_offset(mouse_x, 0)
            # 释放滑块
            action.release().perform()
            time.sleep(2)


    def get_Ori_allCookies(self):
        return self.browser.get_cookies()

    def get_Obj_allCookies(self):
        # 获取cookie
        dictCookies = self.browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def get_Obj_allHeaders(self):
        pass

    def get_URL(self, *browser):
        if(browser==()):
            curUrl = self.browser.current_url()
        else:
            curUrl = browser.browser.current_url()
        return curUrl

    def get_s_v_web_id(self):
        self.handle_SlideCheck()
        pass

    def get_msToken(self):
        pass

    def get_X_Bogus(self):
        pass







from appium import webdriver
import json
json.loads()
caps = {}
caps["platformName"] = "Android"
caps["platformVersion"] = "10.0"
caps["deviceName"] = "HWHLK-H"
caps["ensureWebviewsHavePages"] = True

driver = webdriver.Remote("http://localhost:4723/wd/hub", caps)
TouchAction(driver).tap(x=153, y=837).perform()
TouchAction(driver).tap(x=938, y=2250).perform()
TouchAction(driver).tap(x=139, y=2278).perform()
TouchAction(driver).tap(x=497, y=1483).perform()
TouchAction(driver).tap(x=990, y=153).perform()
desired_caps = self.driver.desired_capabilities()

driver.quit()


'''
    自动化引擎
        视频 筛选 并上传
'''
from globalTools import globalTools
from basement__.Download import Downloader
from basement__ import ContralerDatabase as dbOp
from customFunction__.Poster.poster_video import Poster_Video
from customFunction__.Filter.filter_video import BilibiliFilter
from fake_useragent import UserAgent

def check_hasvideoindb0(checkDate_time, titlePostedList_,item):
    # 过滤数据库中是否存在对应视频方案1
    if(int(item[3]) > checkDate_time and item[0] not in titlePostedList_):
        return True
    else:
        return False

def check_hasvideoindb1(posted_dbOp, checkDate_time,item):
    sql = "SELECT * FROM `postedurldatabase`.`tb_video_posted` WHERE `title`='{}';".format(item[0])
    res = posted_dbOp.getOneDataFromDB(sql)
    # 过滤数据库中是否存在对应视频方案1
    if(int(item[3]) > checkDate_time and not res):
        return True
    else:
        return False

def check_videoRepeat(posted_dbOp, videoInfo):
    sql = "SELECT * FROM `postedurldatabase`.`` WHERE `title`='{}';".format(videoInfo[0])
    res = posted_dbOp.getAllDataFromDB()
    if(res):
        # 重复 返回True
        return True
    else:
        return False

# 爬取源1 bilibili
def run_bilibili(setting):
    # setting['videoDirPath'] = 'E:\\test\\'
    dbOperator = dbOp.Contraler_Database('videodatabase')    # 获取未上传的数据
    posted_dbOp = dbOp.Contraler_Database(databaseName='postedurldatabase')     # 连接上传过的的数据的数据库
    poster = Poster_Video(videoDirPath=setting['videoDirPath'], coverSavedPath=setting['coverSavedPath'], interface='http://121.40.187.51:8088/api/videos_api')
    filter_video = BilibiliFilter()
    # 获取最新爬取下来待上传的视频信息列表
    sql = "SELECT title, avValue, videoUrl, pubdate FROM `videodatabase`.`tb_bilibili_videoInfo`;"
    videoInfoList = dbOperator.getAllDataFromDB(sql)    # 未上传的数据

    i = 1

    # 获取上传过的视频title列表
    titlePostedList = posted_dbOp.getAllDataFromDB("SELECT `title` FROM `postedurldatabase`.`tb_video_posted`;")
    titlePostedList_ = []
    for one in titlePostedList:
        titlePostedList_.append(one[0])

    checkDate_time = int(dbOperator.getOneDataFromDB("SELECT * FROM `videodatabase`.`tb_bilibili_posted_timethenewest`;")[1])

    # 过滤标题操作
    videoInfoList = filter_video.filter_keywordFromTitle(videoInfoList)
    # 针对哔哩哔哩的过滤
    videoInfoList = filter_video.filter_keywordFromTitle4bilibili(videoInfoList)
    if(videoInfoList):
        newestPubdate = checkDate_time  # 上传成功的最近的一次pubdate`bilibilidatabase`.`tb_posted_timethenewest` SET `timethenewest`
        for item in videoInfoList:
            if(check_hasvideoindb1(posted_dbOp, checkDate_time, item)):

                refererUrl = 'https://www.bilibili.com/video/av' + str(item[1])
                vid_headers = {
                    'Origin': 'https://www.bilibili.com',
                    'Referer': refererUrl,
                    'User-Agent':  str(UserAgent().random)
                }
                try:
                    # 视频数据发布时间在当天
                    Downloader.downVideo(urlpath=item[2], name=str(i), dstDirPath=setting['videoDirPath'], headers_=vid_headers)
                    checkIfSuccess = True
                except Exception as e:
                    checkIfSuccess = False
                    print("视频下载出错: ", item)

                if(not checkIfSuccess):
                    # 视频下载出错都不用上传了直接跳过
                    continue

                # 上传
                print("上传视频: ", i)
                try:
                    postResult = poster.post_videoSingle(str(i) + '.mp4', title0=item[0])
                    newestPubdate = int(item[3])   # 对应上传视频的时间戳
                except Exception as e:
                    checkIfSuccess = False
                    print("视频上传出错: ", postResult)
                finally:
                    try:
                        # 上传完删除对应单个视频
                        if(i!=1):
                            globalTools.delVideoSingle(setting['videoDirPath'] + str(i-1) + '.mp4')
                    except Exception as e:
                        print("删除上个视频出错： ", str(i-1) + '.mp4')
                        print(e)

                if(checkIfSuccess):
                    # 更新上传过的数据库 postedurldatabase
                    sql = "INSERT INTO `postedurldatabase`.`tb_video_posted` (`title`) VALUES ('{}');".format(
                        item[0]
                    )
                    posted_dbOp.insertData2DB(sql=sql)
                i = i+1
            else:
                print("不符合条件，无法上传")
                continue
        # 上传一切顺利，更新最新上传的视频的时间戳
        sql_update = "UPDATE `videodatabase`.`tb_bilibili_posted_timethenewest` SET `timethenewest`=\'{}\' WHERE (`id` = '1');".format(
            newestPubdate
        )
        dbOperator.insertData2DB(sql_update)
    else:
        print("数据库为空，无待上传的数据")
    globalTools.finishTask()






from globalTools import globalTools
from basement__ import ContralerDatabase as dbOp
from spider__.selenium_jinritoutiao import Crawler_toutiao
from basement__.ContralSelenium import ReuseChrome
from basement__.ContralerDatabase import Contraler_Database
import time
from customFunction__.Poster import poster_article

def run(setting):
    """
    文章内容爬取需求：
        1 源： 今日头条财经频道 UC浏览器财经频道
        2 清洗： 去除 除p标签和img标签之外的其它标签 去除所有样式属性 img标签保留src属性
        3 字数： 大于500（若是文章内容为图片的话就不要了）
    """
    # setting = {
    #     'databaseName' : 'articledatabase',
    #     'databaseUser' : 'root',
    #     'databasePasswd' : 'root',
    #     "tableName": ['tb_article_toutiao_content'],
    #     'whichKind': 'articles'
    # }
    # setting['sql'] = "select id,comment from ``.``;"
    dbOperator = Contraler_Database(databaseName='data_usable_database')
    sql_get = 'SELECT * FROM `tb_selenium_info` WHERE `id`=\'1\';'
    driver_info = dbOperator.getOneDataFromDB(sql_get)
    browser_toutiao = ReuseChrome(command_executor=driver_info[1], session_id=driver_info[2])
    browser_toutiao.refresh()
    browser_toutiao.get("https://toutiao.com")
    c = Crawler_toutiao()
    time.sleep(5)
    c.click_chaijing(browser_toutiao)
    time.sleep(3)
    c.click_chaijing(browser_toutiao)
    time.sleep(2)
    c.roll_tobottom_method1(browser_toutiao, 600)
    # 1 上传列表
    articleList = c.get_articleContent(browser_toutiao) # 文章元组对象
    # 2 清洗数据并过滤掉不符合条件的数据
    postableList = []
    for article in articleList:
        # 文章字数过滤
        if(len(article[2])>500):
            postableList.append((article[0], article[1], article[2]))
    browser_toutiao.get('https://www.baidu.com')
    # 3 数据插入数据库
    # dbOperator = dbOp.Contraler_Database(databaseName=setting['databaseName'], user=setting['databaseUser'],passwd=setting['databasePasswd'])
    # for article in postableList:
    #     sql = "INSERT INTO `articledatabase`.`tb_article_toutiao_content` (`url`, `title`, `content`) VALUES ('{}', '{}', '{}');".format(
    #         article[0],
    #         article[1],
    #         article[2]
    #     )
    #     dbOperator.insertData2DB(sql=sql)

    # 3 上传数据
    print('可上传文章的数量：', len(postableList))
    poster = poster_article.Poster_Article('http://121.40.187.51:8088/api/article_get')
    poster.post_auto(postableList, 'article')

    # 4 将上传过的数据放到postedurldatabase中
    # postedDBOP = dbOp.Contraler_Database(databaseName='postedurldatabase', user='root', passwd='root')
    # for article in postableList:
    #     sql = "INSERT INTO `postedurldatabase`.`tb_article_posted` (`url`, `title`, `content`) VALUES (\'{}\', \'{}\');".format(
    #         article[0],
    #         article[1],
    #         article[2]
    #     )
    #     postedDBOP.insertData2DB(sql)
    globalTools.finishTask()

# setting = {
#         'databaseName' : 'articledatabase',
#         'databaseUser' : 'root',
#         'databasePasswd' : 'root',
#         "tableName": ['tb_article_toutiao_content'],
#         'whichKind': 'articles'
#     }
# run(setting)


# encoding:utf-8

from aip import AipImageClassify
import requests
import base64
from basement__ import Tools as tools
import os

'''
通用物体和场景识别
'''
class ImgsClassifier():
    __APP_ID = '24654559'
    __API_KEY = 'Pnexi1A99Eobsb1iX7xiBTsE'
    __SECRET_KEY = 'qYYXNhsGLrnAwAWkmHPFOEofQHPRqSYL'

    def __init__(self, crawledDirPath, savedDirPath):
        self.request_url = "https://aip.baidubce.com/rest/2.0/image-classify/v2/advanced_general"
        """ 你的 APPID AK SK """
        self.APP_ID = '24654559'
        self.API_KEY = 'Pnexi1A99Eobsb1iX7xiBTsE'
        self.SECRET_KEY = 'qYYXNhsGLrnAwAWkmHPFOEofQHPRqSYL'
        self.crawledDirPath = crawledDirPath #存放刚刚爬取下来的图片的路径
        self.savedDirPath = savedDirPath
        self.get_imgPathList()

    def get_imgPathList(self):
        # 获取目录下所有文件的路径列表
        imgNameList = os.listdir(self.crawledDirPath)  # 获取目录下的所有文件名
        imgPathList = []
        for imgName in imgNameList:
           item = self.crawledDirPath + '\\' + imgName
           imgPathList.append(item)
        self.imgNameList = imgNameList
        return imgNameList

    @classmethod
    def reconizeImg(cls, imgSrc, request_url = "https://aip.baidubce.com/rest/2.0/image-classify/v2/advanced_general"):
        """识别图片类型 用于视频上传前的判断"""
        # 二进制方式打开图片文件
        f = open(imgSrc, 'rb')
        img = base64.b64encode(f.read())

        params = {"image":img}
        access_token = tools.getAccessToken(cls.__API_KEY, cls.__SECRET_KEY)
        request_url = request_url + "?access_token=" + access_token
        headers = {'content-type': 'application/x-www-form-urlencoded'}

        response = requests.post(request_url, data=params, headers=headers)
        f.close()
        response = response.json()
        lis = []
        if("result" in response):
            result = response["result"]
            result_num = response['result_num']
            # 不收录root只要keyword
            if(result_num >= 2):
                lis.append(result[0]["keyword"])
                lis.append(result[1]["keyword"])
            elif(result_num == 1):
                lis.append(result[0]["keyword"])
        return lis

    def reconizeSingleImgFunc(self, imgName, imgSavedDirPath, request_url = "https://aip.baidubce.com/rest/2.0/image-classify/v2/advanced_general"):
        imgSuffix = '.jpg'
        # imgSrcPath = self.crawledDirPath + imgName + imgSuffix
        imgSrcPath = self.crawledDirPath + imgName

        # 二进制方式打开图片文件
        f = open(imgSrcPath, 'rb')
        img = base64.b64encode(f.read())

        params = {"image":img}
        access_token = tools.getAccessToken(self.API_KEY, self.SECRET_KEY)
        request_url = request_url + "?access_token=" + access_token
        headers = {'content-type': 'application/x-www-form-urlencoded'}


        response = requests.post(request_url, data=params, headers=headers)
        f.close()
        response = response.json()
        print(response)

        if("result" in response):
            result = response["result"]
            result_num = response['result_num']
            # 不收录root只要keyword
            if(result_num >= 2):
                lis = []
                lis.append(result[0]["keyword"].replace('/','（'))
                lis.append(result[1]["keyword"])

                if('/' in lis[0] or '/' in lis[1] or '\\' in lis[0] or '\\' in lis[1]):
                    lis[0] = lis[0].replace('/', '（').replace('\\', '（')
                    lis[1] = lis[1].replace('/', '（').replace('\\', '（')
                name = lis[0] + ' - ' + lis[1]

                reconizedName = '_' + name #采用权重最大头俩个的来命名
            elif(result_num == 1):
                reconizedName = '_' + result[0]["keyword"]
            else:
                reconizedName = ''
            # imgDstPath = imgSavedDirPath + imgName + reconizedName + imgSuffix
            imgDstPath = imgSavedDirPath + imgName.replace('.png','') + reconizedName + '.png'
            try:
                tools.reName(imgSrcPath, imgDstPath)
            except Exception as e:
                print("图片重命名失败：", imgSrcPath, ' -> ',imgDstPath)
                pass
        return response

    def reconizeImgs(self):
        print('待识别的图片总数： ', len(self.imgNameList))
        for imgName in self.imgNameList:
            self.reconizeSingleImgFunc(imgName=imgName, imgSavedDirPath=self.savedDirPath)

    def run(self):
        self.reconizeImgs()


# for imgNum in range(5153, 5782):
#     reconizeFunc(imgNum, request_url)






import os, time, requests, hashlib
from requests_toolbelt import MultipartEncoder
from ..universalTools import tools
from ..Filter import Filter
from ..DatabaserOperator.databaseOperator import dbOperator as dbOp
from io import TextIOWrapper, BytesIO
'''
    Post 视频的类
        参数：
            videoDirPath  处理完的图片目录路径dir   注意这里的路径最后加\\的

'''


class VideoPoster:
    # 传入的dirPath必须是这样的格式： XX:\\XX\\ 最后是有 \\ 的
    def __init__(self, videoDirPath, coverSavedPath, interface='http://121.40.187.51:8088/api/videos_api'):
        self.videoDirPath = videoDirPath
        self.get_videoPathList()
        self.interface = interface
        self.userName = 'qin'
        self.password = 'qin123456'
        self.curDate = str(tools.getCurDate())
        self.key = hashlib.md5(('datapool' + self.userName + self.password + self.curDate).encode('utf-8')).hexdigest()
        # self.videoPathFilteredList = videoPathFilteredList
        self.coverSavedPath = coverSavedPath

    # 获取目录下所有文件的路径列表
    def get_videoPathList(self):
        videoNameList = os.listdir(self.videoDirPath)  # 获取目录下的所有文件名
        videoPathList = []
        for videoName in videoNameList:
            item = self.videoDirPath + '\\' + videoName
            videoPathList.append(item)
        self.videoNameList = videoNameList
        return videoNameList

    # 获取时间戳
    def getCurTime(self):
        return int(round(time.time()))

    def changeMD5(self, videoSrc):
        with open(videoSrc, 'rb') as f:
            md5 = hashlib.md5(f.read()).hexdigest()
        file = open(videoSrc, 'rb').read()
        with open(videoSrc, 'wb') as new_file:
            new_file.write(file + bytes('\0', encoding='utf-8'))  # here we are adding a null to change the file content
            newMD5 = hashlib.md5(open(videoSrc, 'rb').read()).hexdigest()

    # ndarray图像转换io TextIOWrapper
    def translateFrom_Ndarray2TIWrapper(self, ndFile):
        str_encode = ndFile.tostring()
        # str_encode = np.int64(np.all(ndFile[:, :, :3] == 0, axis=2))
        f4 = BytesIO(str_encode)
        f5 = TextIOWrapper(f4)
        return f5

    # 从数据库获取名字
    def getTitleFromDB(self, titleId):
        dbOperator = dbOp("articledatabase")
        sql = "SELECT videoTitle FROM `tb_videodouyingangang` WHERE `id`={};".format(
            titleId
        )
        title = dbOperator.getOneDataFromDB(sql)[0]
        return title

    def post_videoSingle(self, videoName, title0):
        coverSavedPath = self.coverSavedPath
        videoPath = self.videoDirPath + videoName
        print("处理的路径： ", videoPath)
        imgfil = Filter.videoFilter(dirOriPath=self.videoDirPath)
        # imgf = self.translateFrom_Ndarray2TIWrapper(imgfil.getCoverImg(videoPath, coverSavedPath))  # 该获取截图的方法无效——无法上传
        # 判断是否满足条件
        if(imgfil.checkIfTimeLength(videoPath)):
            imgfil.getCoverImg(videoPath, coverSavedPath) # 截图并保存
            imgf = open(coverSavedPath, 'rb')
            videof = open(videoPath, 'rb')
            # title = self.getTitleFromDB(videoName.split('.')[0])

            # 这里传文件的时候用绝对路径传，不然传了之后显示不了
            formData = ({
                "key": self.key,
                "account": self.userName,
                "password": self.password,
                'title': tools.cleanTitle(title0),
                'img': ('cover.jpg', imgf, "image/jpeg"),
                'video': (videoName, videof)
            })
            m = MultipartEncoder(formData)
            headers2 = {
                "Content-Type": m.content_type
            }
            videoPostResult = requests.post(url=self.interface, data=m, headers=headers2)
            imgf.close()
            videof.close()
            return videoPostResult

    # 发送目录下的所有所有视频
    def updateVideos(self):
        # for videoName in self.videoNameList:
        #     self.post_videoSingle(videoName)
        for videoName in self.videoNameList:
            self.post_videoSingle(videoName)




import cv2, os
import numpy as np
from basement__.ContralerImg import Contraler_Img
from basement__ import ContralerDir

'''
    水印识别的类
    识别图片中是否含有指定的模板
'''

class Base_Maskcheck(Contraler_Img):
    @staticmethod
    def method1_cv_reconize(img_gray, template):
        """模板匹配方法1"""
        res = cv2.matchTemplate(img_gray, template, cv2.TM_CCOEFF_NORMED)   # Perform match operations.
        return res

    @staticmethod
    def method2_cv_reconize(img_gray, template):
        """模板匹配方法2"""
        pass

    @staticmethod
    def identifyMask_single_img(img_target_path, imgs_template_path, threshold=0.38):
        '''
        单张图片水印识别
        :param img_target_path: 目标图片绝对路径
        :param img_template_path: 模板图片目录绝对路径 需以\\结尾
        :param threshold: Specify a threshold 0.35本来挺好的
        '''
        # 获取模板目录路径下所有模板绝对
        imgs_templateNameList = os.listdir(imgs_template_path)
        imgs_template_path_list = []
        for templateName in imgs_templateNameList:
            imgs_template_path_list.append(imgs_template_path + templateName)

        # 读取目标图片
        img_target_rgb = cv2.imdecode(np.fromfile(img_target_path, dtype=np.uint8), -1)  # Read the main image

        try:
            img_target_gray = cv2.cvtColor(img_target_rgb, cv2.COLOR_BGR2GRAY)  # Convert it to grayscale
        except Exception as e:
            print('图片转换为灰度失败，请查看原图是否是灰度图片，本功能不应用于灰度图片')
            return {
                "hasWaterMask" : False
            }

        for img_template_path in imgs_template_path_list:
            img_template = cv2.imread(img_template_path, 0)  # Read the template
            w, h = img_template.shape[::-1]  # Store width and heigth of template in w and h

            res = Base_Maskcheck.method1_cv_reconize(img_target_gray, img_template)
            loc = np.where(res >= threshold)  # Store the coordinates of matched area in a numpy array
            x = loc[0]
            y = loc[1]

            # Draw a rectangle around the matched region.
            if (len(x) and len(y)):
                for pt in zip(*loc[::-1]):
                    # pt[0]表示水印位置所在的像素高度
                    cv2.rectangle(img_target_rgb, pt, (pt[0] + w, pt[1] + h), (0, 255, 255), 2)
                    # Show the final image with the matched area.
                    cv2.imwrite("E:\imgs\\toutiao_thumbnail_0to4999_maskreconizeres\\test_001.png", img_target_rgb)
                    waterMaskLocY = pt[1]
                print("发现水印: ", img_target_path, "水印位置： x= ", x, " y= ", y, "对应模板：", img_template_path)
                return {
                    "hasWaterMask" : True,
                    "waterMaskLocY": waterMaskLocY      # 水印位置的y坐标
                }
            else:
                # print('该图片匹配此模板：', img_template_path)
                continue
        print("匹配不到符合当前模板目录下图片模板的水印")
        return {
            "hasWaterMask" : False
       }

    @staticmethod
    def identifyMask_imgs_dir(imgs_dirpath, templates_dirpath, dirpath_hasmask, dirpath_donthasmask):
        '''
            对指定目录下的所有图片进行水印识别处理
        :param imgs_dirpath: 目录绝对路径 最后必须加上\\
        :param templates_dirpath: 模板图片目录的绝对路径 最后必须加上\\
        :param dirpath_hasmask  存放含有水印的图片的目录路径  最后必须加上\\
        :param dirpath_donthasmask  存放无水印的图片的目录路径   最后必须加上\\
        '''
        imgName_list = os.listdir(imgs_dirpath)
        print('待处理图片总数量： ', len(imgName_list))
        hasmask_imgName_list = []
        donthasmask_imgName_list = []
        for imgName in imgName_list:
            imgpath = imgs_dirpath + imgName
            res = Base_Maskcheck.identifyMask_single_img(imgpath, templates_dirpath)
            if(res['hasWaterMask']):
                # 匹配到模板
                hasmask_imgName_list.append(imgName)
                img_hasmask_dst = dirpath_hasmask + imgName
                # 复制到对应目录下
                ContralerDir.Contraler_Dir.copyFile(src=imgpath, dst=img_hasmask_dst)
                # 根据水印位置的裁切 对新目录下的水印图片裁切
                waterMaskY = res["waterMaskLocY"]
                imgHeight = Contraler_Img.get_imgHeight(imgpath)
                cutBottomHeight = imgHeight - waterMaskY
                Contraler_Img.cutting_single_img(img_path=img_hasmask_dst, cutBottomHeight=cutBottomHeight)
            else:
                # 匹配不到模板
                donthasmask_imgName_list.append(imgName)
                img_donthasmask_dst = dirpath_donthasmask + imgName
                # 复制到对应目录下
                ContralerDir.Contraler_Dir.copyFile(src=imgpath, dst=img_donthasmask_dst)





import json
from selenium import webdriver
from time import sleep
import pymysql, time, requests
from basement__.ContralerDatabase import Contraler_Database
from selenium.webdriver.common.keys import Keys
from fake_useragent import UserAgent
from basement__.Encode import Encode
from customFunction__.Cleaner import cleaner_title

class Crawler_Zhihu_Base:
    def __init__(self, url="https://www.zhihu.com/search?q=%E9%85%8D%E8%B5%84&type=content"):
        option = webdriver.ChromeOptions()
        option.add_experimental_option('excludeSwitches', ['enable-automation'])
        option.add_argument('--disable-blink-features=AutomationControlled')
        option.add_experimental_option('useAutomationExtension', False)
        self.browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe",options=option)
        self.browser.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument",{
            "source":"""
                Object.defineProperty(navigator, 'webdriver), {
                    get: (): => undefined
                }
            """
        })
        self.browser.get(url)
        self.url = url
        self.contraler_db = Contraler_Database('data_usable_database')

        print("请先登陆")

    def getCookies(self, browser, *url):
        '''
        类方法，以对象形式输出指定链接返回的cookies
        :param url: 待打开的链接
        :param browser: 浏览器引擎
        :return: cookies对象
        '''
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def pull_down(self):
        """下拉到底"""
        # 下拉到底
        # js = "var q=document.documentElement.scrollTop=20000"
        self.browser.execute_script("""
                        (function () {
                        var y = 0;
                        var step = 100;
                        window.scroll(0, 0);
                        function f() {
                        if (y < document.body.scrollHeight) {
                        y += step;
                        window.scroll(0, y);
                        setTimeout(f, 100);
                        } else {
                        window.scroll(0, 0);
                            document.title += "scroll-done";
                        }
                        }
                        setTimeout(f, 1000);
                        })();
                        """)
        print("下拉中...")
        time.sleep(2)
        # while (True):
        #     if ("scroll-done" in self.browser.title):
        #         break
        #     else:
        #         print("还没有拉到最底端...")
        #         sleep(3)
        n = 0
        while (n <= 30):
            if ("scroll-done" in self.browser.title):
                break
            else:
                print("还没有拉到最底端...")
                sleep(3)
            n = n + 1

    # 提取视频名字的方法
    def videoNameExtract(self, videoSrc):
        return str(videoSrc).split("?")[0].split("/")[-1].replace('.mp4', '')


class Crawler_Zhihu_newest(Crawler_Zhihu_Base):
    def insert_db(self, lis, searchword):
        tag = searchword
        sql = 'INSERT INTO `data_usable_database`.`tb_zhihu_search_articleinfo` (`tag`, `article_url`, `title`, `describe`) VALUES (\'{}\',\'{}\',\'{}\',\'{}\');'
        for item in lis:
            self.contraler_db.insertData2DB(sql.format(
                tag,
                item['article_url'],
                item['title'],
                item['describe']
            ))

    def insert_db_content(self, lis, article_id):
        sql = 'INSERT INTO `data_usable_database`.`tb_zhihu_search_content` (`question_id`, `answer`, `totle_answer`) VALUES (\'{}\',\'{}\',\'{}\');'
        for item in lis:
            self.contraler_db.insertData2DB(sql.format(
                item['article_id'],
                item['content'],
                item['totle_answer']
            ))

    def filter_articleinfo_peizitaolun(self, s):
        filterword = ['配资', '股票', '炒股']
        check = False
        for word in filterword:
            if(word in s):
                check = True
                return check
        return check

    def get_search_articleinfo_result(self, searchword, lis_xpath, item_a_xpath):
        '''
            搜索关键词，获取所有搜索结果
        :param searchword 搜索关键词
        :output lis 文章信息列表
        '''
        lis = []
        time.sleep(3)
        self.pull_down()

        articles_lis = self.browser.find_elements_by_xpath(lis_xpath)
        for article in articles_lis:
            a = article.find_elements_by_xpath(item_a_xpath)
            if(len(a)==1):
                if(self.filter_articleinfo_peizitaolun(a[0].text)):
                    template = {
                        'article_url': '',
                        'title': '',
                        'describe': ''
                    }
                    template['article_url'] = a[0].get_attribute('href')
                    template['title'] = a[0].text
                    template['describe'] = ''
                    lis.append(template)
            elif(len(a)>1):
                for i in a:
                    t = i.text
                    if(t=='' or '更多' in t or '查看全部' in t or '个回答' in t or t=='场外配资' or t=='配资公司' or t=='什么是配资' or t=='配资是什么' or t=='什么是配资怎么配' or t=='配仓'):
                        continue
                    elif(self.filter_articleinfo_peizitaolun(a[0].text)):
                        template = {
                            'article_url': '',
                            'title': '',
                            'describe': ''
                        }
                        template['article_url'] = i.get_attribute('href')
                        template['title'] = i.text
                        template['describe'] = ''
                        lis.append(template)
        return lis

    def click_viewall(self):
        """查看是否有查看所有内容的按钮"""
        try:
            view_all_button = self.browser.find_element_by_xpath("//div[@class='Card ViewAll']//a")
            # 点击
            """
                注意selenium的a标签的点击方法不能直接用click() 要用send_keys方法
            """
            view_all_button.send_keys(Keys.ENTER)
        except Exception as e:
            pass

    def close_loginform(self):
        """关闭登录表单"""
        try:
            login_form = self.browser.find_element_by_xpath("//button[@class='Button Modal-closeButton Button--plain']")
            login_form.send_keys(Keys.ENTER)
        except Exception as e:
            pass


    def get_search_content_result(self, url):
        """
        获取指定知乎链接下所有回答的内容
        :output lis 回答内容的字典列表，包括url
        """
        lis = []
        if('question' in url):
            article_id = self.contraler_db.getOneDataFromDB("SELECT `id` FROM `tb_zhihu_search_articleinfo` WHERE `article_url`=\'{}\'".format(url))[0]
            self.browser.get(url)
            self.close_loginform()
            self.pull_down()
            self.click_viewall() # 点击查看更多
            self.close_loginform()
            self.pull_down()
            totle_answer = self.browser.find_element_by_xpath("//div[@class='List-header']//h4//span").text.replace(' 个回答','')
            contentslist = self.browser.find_elements_by_xpath("//div[@role='list']//div[@class='List-item']//div[@class='ContentItem AnswerItem']")
            for item in contentslist:
                template = {
                    'article_id': int(article_id),
                    'content': '',
                    'totle_answer': totle_answer
                }
                s = ''
                pList = item.find_elements_by_xpath(".//div[@class='RichContent RichContent--unescapable']//span[@class='RichText ztext CopyrightRichText-richText css-hnrfcf']//p")
                for p in pList:
                    s = s + '\n' + p.text
                template['content'] = s
                lis.append(template)
        return lis

    def run_articleInfo(self, searchword='配资'):
        # 抓取文章信息数据
        lis = self.get_search_articleinfo_result(
            searchword=searchword,
            lis_xpath="//div[@role='list']//div[@role='listitem']",
            item_a_xpath=".//a"
        )
        # 将文章数据上传到数据库
        self.insert_db(lis, searchword=searchword)
    def run_articleContent_special_qsid(self, questionId, pulldowntimes):
        url_item = self.contraler_db.getOneDataFromDB('SELECT `id`,`article_url` FROM `tb_zhihu_search_articleinfo` WHERE `id`=\'{}\';'.format(questionId))
        content_lis = self.get_search_content_result(url_item[1])
        self.insert_db_content(lis=content_lis, article_id=url_item[0])

    def run_articleContent(self, tag):
        url_lis = self.contraler_db.getAllDataFromDB('SELECT `id`,`article_url` FROM `tb_zhihu_search_articleinfo` WHERE `tag`=\'{}\';'.format(tag))
        for item in url_lis:
            content_lis = self.get_search_content_result(item[1])
            self.insert_db_content(lis=content_lis, article_id=item[0])
            time.sleep(3)

    def run_articleInfo_gupiaopeizi(self, searchword='股票配资', url='https://www.zhihu.com/topic/19958084/hot'):
        """注意： 数据库里若是知乎内容部分question_id为198之后的数据中若是有question_id<198，说明对应链接与前面重复了"""
        lis = self.get_search_articleinfo_result(
            searchword,
            lis_xpath="//div[@role='list']//div[@class='List-item TopicFeedItem']",
            item_a_xpath=".//h2//a"
        )
        self.insert_db(lis=lis,searchword=searchword)




from ..DatabaserOperator import databaseOperator as dbOp
'''
    输入： 段落字符串
    输出： 段落字符串
    清洗这里是通用的
'''
class Cleaner():
    def __init__(self):
        self.filterWord = [
            '●', '但是，', '所以，', '再说，', '虽然说，', '另外，', '最后，', '而且，',
            '其次，', '首先，', '再者，', '同时，', '不过，', '当然，', '当然啦，', '那么，', '虽然，',
            '其实，', '通常，','接着，','综上所述，', '因此，', '、'
        ]
        self.alphabeticList = [
            'A','B','C','D','E','F','G','H'
        ]
        self.numList1 = ['一','二','三','四','五','六','七','八','九','十']
        self.numList2 = ['第一','第二','第三','第四','第五','第六','第七','第八','第九','第十']
        for i in range(1, 10):
            self.filterWord.append(str(i) + '.')
            self.filterWord.append(str(i) + '。')
            self.filterWord.append(str(i) + '，')
            self.filterWord.append(str(i) + '、')
            self.filterWord.append(str(i) + '：')
            self.filterWord.append('（' + str(i) + '）')
            self.filterWord.append(str(i) + '）')
            self.filterWord.append('(' + str(i) + ')')
            self.filterWord.append('[' + str(i) + ']')
            self.filterWord.append('【' + str(i) + '】')
        for item in self.numList1:
            self.filterWord.append(item + '、')
            self.filterWord.append(item + '。')
            self.filterWord.append('(' + item + ')')
        for item in self.numList2:
            self.filterWord.append(item + '、')
            self.filterWord.append(item + '，')
        for item in self.alphabeticList:
            self.filterWord.append(item + '.')
        self.filterWord = tuple(self.filterWord)

        # 从数据库获取股票名字和代码
        sql = "select name, code from `tb_namecode`"
        dbOperator = dbOp.dbOperator('stocksnamecode')
        self.stocksNameCodeList = dbOperator.getAllDataFromDB(sql)
        dbOperator.closeDb()
        del dbOperator

    # 清洗掉空格和结尾的空格
    def delSpace(self, paragraph):
        return paragraph.replace("\r", "").replace("\n", "").replace("\t", "").replace("\xa0", "").replace("\u3000","")

    # 删除开头的序号 如 1. 2. 1、 2、 1， 2，
    def delHeaderNum(self, paragraph):
        for filterword in self.filterWord:
            if(paragraph.startswith(filterword)):
                paragraph = paragraph[len(filterword):]
                break
            # paragraph = paragraph.lstrip(filterword)
        return paragraph

    # 删除股票名字和股票代码
    def delStocksNameAndCode(self, paragraph):
        for item in self.stocksNameCodeList:
            li = []
            li.append(item[0] + "（{}）".format(item[1]))
            li.append("（{}）".format(item[0]))
            li.append("（{}）".format(item[1]))
            li.append(item[0])
            li.append(item[1])
            for stock in li:
                paragraph = paragraph.replace(stock, "")  # 删除股票关键字（包括中文及代买关键字）
        return paragraph

    # 删除杂七杂八的不相关的文字 如： 图9-5
    def delIrrelevantWord(self, paragraph):
        self.irrelevantWordList = ['图']
        for irrelevantWord in self.irrelevantWordList:
            paragraph = paragraph.replace(irrelevantWord, "")
        return paragraph

    # 集成操作方法， 输出的结果为最终清洗完成的结果
    def integratedOp(self, paragraph):
        # 清序号
        temp = self.delHeaderNum(paragraph)
        # 清空格
        temp = self.delSpace(temp)
        # 清序号
        temp = self.delHeaderNum(temp)
        # 清股票
        # temp = self.delStocksNameAndCode(temp)
        # 清不相关的文字
        result = self.delIrrelevantWord(temp)
        return result


import os, time, re, requests
import hashlib, base64
from selenium import webdriver

# 获取当前根目录路径
def getCurOriPath():
    return os.path.abspath(os.path.dirname(__file__))

# 清洗掉开头空格和结尾的空格
def delSpace(paragraph):
    # return paragraph.replace("\r", "").replace("\n", "").replace("\t", "").replace("\xa0", "").replace("\u3000","")
    return paragraph.strip()

def finishTask():
    print("流程结束，单次任务结束（爬取、处理、上传数据， 对应数据库数据的清空以及posturldatabase数据库的更新）")

# 清空指定目录下所有文件
def clearDirFiles(dirPath):
    lis = os.listdir(dirPath)
    for i in lis:
        os.remove(dirPath + i)

# 删除指定文件
def delVideoSingle(filePath):
    os.remove(filePath)


# 获取当前日期
def getCurDate():
    return time.strftime("%Y%m%d", time.localtime())

# 返回指定日期时间戳 时间格式 '%Y%m%d %H:%M:%S' 20210924 00：00：00 该方法用于哔哩哔哩时间的判断
def getSecondByDate(date):
    b = time.strptime(date, '%Y%m%d %H:%M:%S')
    return time.mktime(b)

# 通过selenium获取指定信息的类
class GetParams_Selenium:
    def __init__(self, *driverPath):
        if(driverPath!=()):
            option = webdriver.ChromeOptions()
            option.add_experimental_option('excludeSwitches', ['enable-automation'])
            option.add_experimental_option('useAutomationExtension', False)
            self.browser = webdriver.Chrome(driverPath, options=option)


    def getCookies(self, *url, browser):
        '''
        类方法，以对象形式输出指定链接返回的cookies
        :param url: 待打开的链接
        :param browser: 浏览器引擎
        :return: cookies对象
        '''
        # 获取cookie
        dictCookies = browser.get_cookies()
        cookies = {}
        for item in dictCookies:
            key = item['name']
            cookies[str(key)] = str(item['value'])
        return cookies

    def del_all_cookies(self):
        self.browser.delete_all_cookies()

    def get_params(self, url, *browser):
        if(browser==()):
            self.browser.get(url)
            time.sleep(1)
            cookies = self.getCookies(url, self.browser)
            headers = {}
            return {
                'cookies': cookies,
                'headers': headers
            }
        else:
            browser.get(url)
            time.sleep(1)
            cookies = self.getCookies(url, browser)
            headers = {}
            return {
                'cookies': cookies,
                'headers': headers
            }


    def closeBrowser(self, *browser):
        '''
        关闭指定浏览器，若浏览器为None则关闭对象浏览器
        :param browser:
        :return: None
        '''
        if(browser==()):
            self.browser.close()
        else:
            browser.close()


# 字符串正则处理类
class Handler_String_ByRe:
    def extract_StrByRe(self, string, patternStr=r'[[](.*?)[]]'):
        '''
        正则匹配字符串获取指定内容(匹配[]包括这的内容)
        :param string: 待匹配的字符串
        :param patternStr: 正则表达式模式
        :return: 列表
        '''
        pattern = re.compile(patternStr, re.S)  # 最小匹配
        return re.findall(pattern, string)



class Handle_PackageInfo:
    '''
        处理报文信息 如 cookie和headers字符串和对象的转换
    '''
    def translate_Cookies_Row2Obj(self, cookiesRow):
        cookieList = cookiesRow.split(";")
        self.cookies = {}
        for cookieItem in cookieList:
            i = cookieItem.strip().split("=")
            k = i[0]
            v = i[1]
            self.cookies[k] = v
        return self.cookies

    def translate_Headers_Row2Obj(self, headersRow):
        headerList = headersRow.split('\n')
        self.headers = {}
        for headerItem in headerList:
            i = headerItem.strip().split(":")
            if (i != ['']):
                k = i[0]
                v = i[1]
                self.headers[k] = v
        return self.headers


# 加密类
class Encode:
    # 同一输出类型为str
    def bytes2str(self, b):
        return str(b, encoding='utf-8')

    def str2bytes(self, s):
        return bytes(s, encoding='utf-8')

    def encodeByMd5(self, s):
        return hashlib.md5(s.encode(encoding='utf-8')).hexdigest()

    # base64输出的为bytes类型 要转化为字符串
    def encodeByBase64(self, s):
        res = base64.encodebytes(s).strip()
        # 转换为字符串
        res = self.bytes2str(res)
        return res

    def encode0(self,s):
        return self.encodeByBase64(self.str2bytes(self.encodeByMd5(s)))

# 目录处理类
class Contraler_Dir:
    # 获取当前根目录路径
    def getCurOriPath(self):
        return os.path.abspath(os.path.dirname(__file__))

    # 判断目录是否存在，不存在则创建
    def checkACreateDir(self,dirPath):
        exist = os.path.exists(dirPath)
        if (not exist):
            os.makedirs(dirPath)
        else:
            pass
        pass

    # 清空指定目录下所有文件
    def clearDirFiles(self, dirPath):
        lis = os.listdir(dirPath)
        for i in lis:
            os.remove(dirPath + i)

# 时间处理类
class Contraler_Time:
    # 获取当前日期
    def getCurDate(self):
        return time.strftime("%Y%m%d", time.localtime())

    # 返回指定日期时间戳 时间格式 '%Y%m%d %H:%M:%S' 20210924 00：00：00 该方法用于哔哩哔哩时间的判断
    def getSecondByDate(self, date):
        b = time.strptime(date, '%Y%m%d %H:%M:%S')
        return time.mktime(b)

# 下载类
class Downloader:
    # 已有图片链接下载图片的方法
    def download_img(self, urlpath, imgname, dstDirPath):
        '''
        图片下载
        :param urlpath: 可下载的图片资源链接
        :param imgname: 保存图片为指定名字
        :param dstDirPath: 图片保存目录位置，注意：最后要有\\
        :return:
        '''
        r = requests.get(urlpath)
        img = r.content  # 响应的二进制文件
        with open(dstDirPath + str(imgname) + '.png', 'wb') as f:  # 二进制写入
            f.write(img)

    # 下载视频
    def downVideo(self, urlpath, name, dstDirPath, headers_):
        # 获取当前日期
        r = requests.get(urlpath, verify=False, headers=headers_, timeout=30)
        video = r.content  # 响应的二进制文件
        with open(dstDirPath + str(name) + '.mp4', 'wb') as f:  # 二进制写入
            f.write(video)
        r.close()  # 关闭很重要，确保不要过多的连接


import cv2
import numpy as np

'''
    
'''
class SliderChecker_Douyin:
    def __init__(self, templatePath='E:\Projects\packageDIY\\videoRef\\assets\\template.png', targetPath='E:\Projects\packageDIY\\videoRef\\assets\\target.png'):
        self.templatePath = templatePath
        self.targetPath = targetPath


    # 滑块验证 输出水平移动的距离 计算坐标的偏移量
    # templatePath 滑块 , targetPath 背景模板
    def getMovement_sliderCheck_douyin(self, elWidth):
        '''
            滑块验证 输出水平移动的距离 计算坐标的偏移量
            注意：这一个算法没问题，只是可能需要试几次才有一次正确的
        :param elWidth: 背景图在网页上显示的宽度，用于比例计算实际滑动距离
        :param templatePath: 滑块图片路径
        :param targetPath: 背景图片路径
        :return: 偏移量
        '''
        target_rgb = cv2.imdecode(np.fromfile(self.targetPath, dtype=np.uint8), -1)
        target_gray = cv2.cvtColor(target_rgb, cv2.COLOR_BGR2GRAY)
        template_rgb = cv2.imread(self.templatePath, 0)
        ori_width = target_rgb.shape[1]
        ori_height = target_rgb.shape[0]
        res = cv2.matchTemplate(target_gray, template_rgb, cv2.TM_CCOEFF_NORMED)
        value = cv2.minMaxLoc(res)
        # (因为下载的图片跟网页上的图片是有缩放的)所以要根据比例计算网页端需要移动的偏移量
        effectiveVal = value[2][0] * elWidth / ori_width

        print('effectiveVal:', effectiveVal)
        print('value2:', value[2][0])
        print('value3:', value[3][0])
        print('res:', res.shape)

        return effectiveVal

    # 滑块验证 输出水平移动的距离 计算坐标的偏移量
    # templatePath 滑块 , targetPath 背景模板
    @staticmethod
    def getMovement_sliderCheck_douyin_static(templatePath, targetPath, elWidth):
        '''
            滑块验证 输出水平移动的距离 计算坐标的偏移量
            注意：这一个算法没问题，只是可能需要试几次才有一次正确的
        :param elWidth: 背景图在网页上显示的宽度，用于比例计算实际滑动距离
        :param templatePath: 滑块图片路径
        :param targetPath: 背景图片路径
        :return: 偏移量
        '''
        target_rgb = cv2.imdecode(np.fromfile(targetPath, dtype=np.uint8), -1)
        target_gray = cv2.cvtColor(target_rgb, cv2.COLOR_BGR2GRAY)
        template_rgb = cv2.imread(templatePath, 0)
        ori_width = target_rgb.shape[1]
        ori_height = target_rgb.shape[0]
        res = cv2.matchTemplate(target_gray, template_rgb, cv2.TM_CCOEFF_NORMED)
        value = cv2.minMaxLoc(res)
        # (因为下载的图片跟网页上的图片是有缩放的)所以要根据比例计算网页端需要移动的偏移量
        effectiveVal = value[2][0] * elWidth / ori_width

        print('effectiveVal:', effectiveVal)
        print('value2:', value[2][0])
        print('value3:', value[3][0])
        print('res:', res.shape)

        return effectiveVal







import setuptools

setuptools.setup(
    name="auto_datahandler", # Replace with your own username  #自定义封装模块名与文件夹名相同
    version="0.0.10", #版本号，下次修改后再提交的话只需要修改当前的版本号就可以了
    updatetime="20211210",
    author="唐国钦", #作者
    author_email="dreamertgq@163.com", #邮箱
    description="数据（段落、评论、图片、视频）的自动化处理 在之前项目packageDIY的基础上进行代码的简化和整合", #描述
    long_description='数据（段落、评论、图片、视频）的自动化处理 在之前项目packageDIY的基础上进行代码的简化和整合', #描述
    long_description_content_type="text/markdown", #markdown
    url="https://github.com/Tommy-pixels/auto_datahandler", #github地址
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License", #License
        "Operating System :: OS Independent",
    ],
    REQUIRED = [
        'requests', 'pymysql', 'baidu-aip','beautifulsoup4','fake-useragent','matplotlib',
        'opencv-python','openpyxl','Pillow','PyAutoGUI','requests','requests-toolbelt',
        'Scrapy','scrapy-splash','selenium','xlrd', 'moviepy', 'openpyxl'
    ],
    python_requires='>=3.6.8',  #支持python版本
)


import time

"""
    时间（时间戳、日期、年月日等）处理相关的基类和方法
"""
# 时间处理类
class Contraler_Time:
    @staticmethod
    def getMilliSecond():
        '''
        返回毫秒级时间戳
        '''
        return int(round(time.time() * 1000))

    # 获取当前日期
    @staticmethod
    def getCurDate(formatStr:str):
        '''
        获取当前日期
        :param formatStr: 指定格式 如 "%Y%m%d"
        :return:
        '''
        return time.strftime(formatStr, time.localtime())

    # 返回指定日期时间戳 时间格式 '%Y%m%d %H:%M:%S' 20210924 00：00：00 该方法用于哔哩哔哩时间的判断
    @staticmethod
    def getSecondByDate(date):
        b = time.strptime(date, '%Y%m%d %H:%M:%S')
        return time.mktime(b)


import math
import time
import execjs


class Executor_Javascript:
    """执行javascript代码/文件 输出结果"""
    @classmethod
    def execute_jsfile(cls, filepath):
        """
        输出js执行环境
        :param filepath js文件路径
        :output
        """
        js = ''
        with open(filepath, 'r', encoding="utf-8") as f:
            jsLis = f.readlines()
            for li in jsLis:
                js = js + li
        js_executor = execjs.compile(js)
        return js_executor

    @classmethod
    def execute_jsfunc(cls, js_executor, funcName, *funcArgs):
        """
        用execjs.compile().call(函数名, 函数参数)方法在js执行函数里执行指定函数
        """
        return js_executor.call(funcName, funcArgs)


from globalTools import globalTools
from basement__ import ContralerDatabase as dbOp
from customFunction__.Poster import poster_question

def run():
    dbOperator = dbOp.Contraler_Database(databaseName='data_usable_database')
    questionList = dbOperator.getAllDataFromDB(sql='SELECT * FROM data_usable_database.tb_zhihu_search_question;')
    QAList = []
    for question in questionList:
        answerList = []
        data = dbOperator.getAllDataFromDB(sql='SELECT `answer` FROM data_usable_database.tb_zhihu_search_content WHERE `question_id`={};'.format(question[0]))
        if(not data):
            continue
        for ans in data:
            answerList.append(ans[0])
        QAList.append((question[2], answerList))
    # 上传数据
    poster = poster_question.Poster_Question('http://121.40.187.51:8088/api/question_get')
    poster.post_auto(QAList, 'question')
    globalTools.finishTask()




import json
import requests, hashlib
from basement__.ContralerTime import Contraler_Time
from basement__.ContralerDatabase import Contraler_Database
from requests_toolbelt import MultipartEncoder


class Base_Poster:
    interface = ''
    userName = 'qin'
    password = 'qin123456'
    contralTime = Contraler_Time()
    curDate = str(contralTime.getCurDate(formatStr="%Y%m%d"))
    key = hashlib.md5(('datapool' + userName + password + curDate).encode('utf-8')).hexdigest()
    dbOperator = Contraler_Database(databaseName='postedurldatabase')


    # 上传单个数据列表的方法
    @staticmethod
    def poster(postableData, interface):
        '''
        上传单条数据的方法
        :param postableData:
        :param interface: 接口
        :return:
        '''
        # 这里传文件的时候用绝对路径传，不然传了之后显示不了
        formData = (postableData)
        m = MultipartEncoder(formData)
        headers2 = {
            "Content-Type": m.content_type
        }
        paragraphPostResult = requests.post(url=interface, data=m, headers=headers2)
        return paragraphPostResult


    def post_auto(self, effectiveDataList, whichKind):
        for item in effectiveDataList:
            if (whichKind == 'keyParagraph'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'content': item[0],
                    'keyword': item[1],
                    'rekeyword': '配资'
                }
                self.poster(postableData=postableData, interface=self.interface)
            elif (whichKind == 'relativeParagraph'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'content': item[0],
                    'keyword': item[1]
                }
                self.poster(postableData=postableData, interface=self.interface)
            elif (whichKind == 'articleComment'):
                comment = item[0].strip()
                sql = "SELECT * FROM `postedurldatabase`.`tb_comment_posted` where `comment` = \'{}\';".format(comment)
                if (self.dbOperator.getOneDataFromDB(sql)):
                    # 数据库中存在对应评论
                    continue
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'comment': item[0].replace(item[1], '股票').replace('&nbsp;', ' ')
                }
                self.poster(postableData=postableData, interface=self.interface)
                # 更新数据库
                self.update_postedurldb(item)
            elif(whichKind == 'article'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'title': item[1],
                    'content': item[2]
                }
                self.poster(postableData=postableData, interface=self.interface)
            elif (whichKind == 'question'):
                postableData = {
                    "key": self.key,
                    "account": self.userName,
                    "password": self.password,
                    'question': str(item[0]),
                    'answer': json.dumps(item[1])
                }
                res = self.poster(postableData=postableData, interface=self.interface)
                print(res.text)



import re
from ..DatabaserOperator import databaseOperator as dbOp
from ..Cleaner import Cleaner
'''
    输入： 元组
    输出： 判断结果 布尔值
    注意： 筛选这里根据数据库表结构的不同，筛选方法有所差异
'''
class Filter():
    # 输入 段落
    def filter_BetweenNumberOfWords(self, paragraph, whichKind):
        if(whichKind == 'relativeParagraph'):
            if (len(paragraph) > 125 and len(paragraph) < 250):
                return True
            else:
                return False
        elif(whichKind == 'keyParagraph'):
            if (len(paragraph) > 50 and len(paragraph) < 250):
                return True
            else:
                return False
        elif(whichKind == 'articleComment'):
            if (len(paragraph) > 45 and len(paragraph) < 250):
                return True
            else:
                return False

    # -> 关联段落判断关键词
    def checkIfHasKeyword_comment(self, paragraph, keyword):
        if(keyword in paragraph):
            # 关键字存在而且段落字数符合要求返回True
            return True
        else:
            return False

    # -> 评论内容的集成操作方法
    def integratedOp4List_comment(self, commentList, keywordList):
        result = []
        cleanerInstance = Cleaner.Cleaner()
        for item in commentList:
            # 1 进行清洗操作(评论不用清洗 去除两边空格就可了)
            item[1] = cleanerInstance.integratedOp_comment(item[1])
            item[1] = item[1].strip('1.').strip('2.').strip('3.').strip('4.').strip('5.').strip('6.').strip('7.').strip('8.').strip()
            for keyword in keywordList:
                # 2 根据字符串长度25-250间进行筛选
                if (not self.filter_BetweenNumberOfWords(item[1], whichKind='articleComment')):
                    # 段落字数再25-250间，有用可传
                    continue
                # 3 对每个段落进行关键词筛选 若有关键词则跳出当前循环
                if (self.checkIfHasKeyword_comment(item[1], keyword)):
                    # 4 根据筛选结果导入可上传的数据
                    result.append(
                        (
                            item[1],  # 评论内容
                            keyword,  # 相关关键词
                        )
                    )
                    break
        return result

class Filter_Posted():
    def __init__(self):
        self.dbOperator = dbOp.dbOperator(databaseName='postedurldatabase')
        self.cleaner = Cleaner.Cleaner()

    # comment 为待处理上传的数据
    def filterPosted(self, comment):
        comment = self.cleaner.integratedOp_comment(comment)
        comment = comment.strip()  # 清除一下左右空格
        sql = "SELECT * FROM `postedurldatabase`.`tb_comment_posted` where `comment` = \'{}\';".format(comment)

        check = self.dbOperator.getOneDataFromDB(sql)
        if(check):
            return True
        else:
            return False

    def run(self, dataOriList):
        result = []
        for item in dataOriList:
            if(not self.filterPosted(comment=item[1])):
                # 没上传过，加入结果
                result.append(item)
        return result



"""
    抖音视频 快手视频 的过滤爬取上传自动化流程
        注意做好滑块的验证功能：
            该功能下注意图片的缩放， 网页图片的显示的尺寸同下载下来的图片的尺寸是有差别的，导致滑块识别出来的值有差异
        滑块模板和背景图片下载下来后通过 opencv进行模板匹配获得滑动的距离
        最后再利用
            selenium的控制
            pyguiauto的鼠标模拟

        数据要抓当天的而且去除重复
            过滤条件要判断发布的时间是否是当天的
                抖音有一个发布时间的标签，显示了视频的发布时间如 2021-09-17
                快手没有发布时间，显示的只有 n小时前 n天前 n周前 n月前，一般不会有1天前，只有两天前，一天内的都用x小时表示，如 10小时前
            去除重复
                对于抖音:
                    通过在本地数据库 postedurldatabase 存储上传过的视频的title，每次上传时和本地数据库的title比较进行判断
                对于快手：
                    通过在本地数据库 postedurldatabase 存储对应视频链接的clientCacheKey进行判断，每次上传时和本地数据库的clientCacheKey比较进行判断
        爬取的url
        抖音：{
            # "股市-综合排序-发布时间": "https://www.douyin.com/search/%E8%82%A1%E5%B8%82?publish_time=0&sort_type=0&source=normal_search&type=video"
            "股-最新发布-一天内": "https://www.douyin.com/search/%E8%82%A1?publish_time=1&sort_type=2&source=normal_search&type=video"
            "杠杆-最新发布-一天内": "https://www.douyin.com/search/%E6%9D%A0%E6%9D%86?publish_time=1&sort_type=2&source=normal_search&type=video"
            "股票-最新发布-一天内": "https://www.douyin.com/search/%E8%82%A1%E7%A5%A8?publish_time=1&sort_type=2&source=normal_search&type=video"
            "涨跌-最新发布-一天内": "https://www.douyin.com/search/%E6%B6%A8%E8%B7%8C?publish_time=1&sort_type=2&source=normal_search&type=video"
            "大盘-最新发布-一天内": "https://www.douyin.com/search/%E5%A4%A7%E7%9B%98?publish_time=1&sort_type=2&source=normal_search&type=video"
            "B股-最新发布-一天内": "https://www.douyin.com/search/B%E8%82%A1?publish_time=1&sort_type=2&source=normal_search&type=video"
            "短线-最新发布-一天内": "https://www.douyin.com/search/%E7%9F%AD%E7%BA%BF?publish_time=1&sort_type=2&source=normal_search&type=video"
            "指数-最新发布-一天内": "https://www.douyin.com/search/%E6%8C%87%E6%95%B0?publish_time=1&sort_type=2&source=normal_search&type=video"
        }
        快手：{
            "股": "https://www.kuaishou.com/search/video?searchKey=%E8%82%A1"
            ""
        }

    关于滑块验证：
        + 百度查了好久就只有两种方法,但是每一种管用
            1 用cv2模板匹配的方式获得滑动距离，然后selenium再滑动对应元素
                这里涉及可能匹配错误，导致获得错误的距离，即便匹配正确获得正确的滑动距离，好像也是会被检测到导致滑动失败
            2 添加option
                · 针对旧版本 chrome
                option = webdriver.ChromeOptions()
                option.add_experimental_option('excludeSwitches', ['enable-automation'])
                option.add_experimental_option('useAutomationExtension', False)
                self.browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", options=option)
                · 针对新版本 chrome
                option = webdriver.ChromeOptions()
                option.add_experimental_option('excludeSwitches', ['enable-automation'])
                option.add_experimental_option('useAutomationExtension', False)
                self.browser = webdriver.Chrome(executable_path="E:\Projects\webDriver\\chrome\\chromedriver.exe", options=option)
                self.browser.execute_cdp_cmd('Page.addScriptToEvaluateOnNewDocument', {
                    'source': 'Object.defneProperty(navigator, "webdriver", {get: () => Chrome})'
                })
            上面这几种方法好像以前能用，反正现在我用了没一点效果
        + 最后反而再哔哩哔哩某个没用的视频的评论里找到了有效的方法
            option = webdriver.ChromeOptions()
            option.add_experimental_option('excludeSwitches', ['enable-automation'])
            option.add_argument('--disable-blink-features=AutomationControlled')

"""


'''
    此处处理上传缩略图--今日头条
'''
from basement__ import ContralerDatabase, Download
from customFunction__.imgref import classifier, processing
from customFunction__.imgref import maskcheck
from customFunction__.Poster import poster_img
from customFunction__.imgref import processing

from customFunction__.Poster import poster_img

# config = {
#     'template_dirpath': 'E:\\Projects\\auto_datahandler\\resource\\mask_templates\\toutiao_templates\\',
#     'thumbnail_dirpath': 'E:\\imgs\\toutiao_thumbnail_0to4999\\',
#     'thumbnail_dirpath_reconized': 'E:\\imgs\\toutiao_thumbnail_0to4999_reconized\\',
#     'thumbnail_dirpath_hasmask': 'E:\\imgs\\toutiao_thumbnail_0to4999_hasmask\\',
#     'thumbnail_dirpath_donthasmask': 'E:\\imgs\\toutiao_thumbnail_0to4999_donthasmask\\',
#     'thumbnail_dirpath_maskreconizeres': 'E:\\imgs\\toutiao_thumbnail_0to4999_maskreconizeres',# 水印识别结果
#     'thumbnail_dirpath_tobeposted': 'E:\\imgs\\toutiao_thumbnail_0to4999_tobeposted\\', # 待上传的图
#
#     'contentimg_dirpath': 'E:\\imgs\\toutiao_contentimg_5000toend\\',
#     'contentimg_dirpath_reconized': 'E:\\imgs\\toutiao_contentimg_5000toend_reconized\\',
#     'contentimg_dirpath_hasmask': 'E:\\imgs\\toutiao_contentimg_dirpath_hasmask\\',
#     'contentimg_dirpath_donthasmask': 'E:\\imgs\\toutiao_contentimg_dirpath_donthasmask\\',
#     'contentimg_dirpath_maskreconizeres': 'E:\\imgs\\toutiao_contentimg_5000toend_maskreconizeres'
# }

config = {
    'template_dirpath': 'E:\\Projects\\auto_datahandler\\resource\\mask_templates\\toutiao_templates\\',
    'thumbnail_dirpath': 'E:\\toutiao\\origin\\',
    'thumbnail_dirpath_reconized': 'E:\\toutiao\\reconized\\',
    'thumbnail_dirpath_hasmask': 'E:\\toutiao\\hasmask\\',
    'thumbnail_dirpath_donthasmask': 'E:\\toutiao\\donthasmask\\',
    'thumbnail_dirpath_maskreconizeres': 'E:\\toutiao\\maskreconizeres',# 水印识别结果
    'thumbnail_dirpath_tobeposted': 'E:\\toutiao\\tobeposted\\', # 待上传的图
}

class Tools:
    contraler_db = ContralerDatabase.Contraler_Database('imgsdatabase')
    downloader = Download.Downloader()
    imgclassifier_thumbnail = classifier.ImgsClassifier(crawledDirPath=config['thumbnail_dirpath'], savedDirPath=config['thumbnail_dirpath_reconized'])
    # imgclassifier_contentimg = classifier.ImgsClassifier(crawledDirPath=config['contentimg_dirpath'], savedDirPath=config['contentimg_dirpath_reconized'])
    maskchecker = maskcheck.Base_Maskcheck()


tableName = 'tb_contentimg_toutiao'
toutiao_imgurl_list = Tools.contraler_db.getAllDataFromDB('select `origin_pic_path` from `{}`;'.format(tableName))
print(tableName, ' 数据总数目： ', len(toutiao_imgurl_list))




print('------------------ 处理今日头条图片（前5k张）传至缩略图 -------------------------')
thumbnail_toutiao_0to4999_list = toutiao_imgurl_list[0:5000]
print("缩略图_头条_数目： ", len(thumbnail_toutiao_0to4999_list))
print('读取 例子 ', toutiao_imgurl_list[0])
# print("1 下载图片到目录")
# for i, url in enumerate(thumbnail_toutiao_0to4999_list):
#     try:
#         Tools.downloader.download_img(url[0], i, config['thumbnail_dirpath'])
#     except Exception as e:
#         print(i, ' ', url)
# 图片下载已经完成，人工筛选 可上传的占比约一半
# print('2 图片识别-命名')
# Tools.imgclassifier_thumbnail.run()

# print('3 水印识别及处理')
# Tools.maskchecker.identifyMask_imgs_dir(imgs_dirpath=config['thumbnail_dirpath_reconized'], templates_dirpath=config['template_dirpath'], dirpath_hasmask=config['thumbnail_dirpath_hasmask'], dirpath_donthasmask=config['thumbnail_dirpath_donthasmask'])

# print('4 将图片整成缩略图')
# processer0 = processing.Imgs2ThumbnailByDir(thumbnailOriDirPath=config['thumbnail_dirpath_donthasmask'],thumbnailDstDirPath=config['thumbnail_dirpath_tobeposted'])
# processer0.cutOff2ThumbnailByDir()

# print('5 上传无水印的缩略图')
# poster = poster_img.Poster_Imgs(imgDirPath=config['thumbnail_dirpath_tobeposted'])
# poster.updateImgsThumbnail()

print('------------------ 处理今日头条图片（5k张之后的）传至内容图 -------------------------')
# contentimg_toutiao_5000toend_list = toutiao_imgurl_list[5000:]
# print("内容图_头条_数目： ", len(contentimg_toutiao_5000toend_list))
# print("1 下载图片到目录")
# for i, url in enumerate(contentimg_toutiao_5000toend_list):
#     try:
#         Tools.downloader.download_img(url[0], i, config['contentimg_dirpath'])
#     except Exception as e:
#         print(i, ' ', url)
# 图片下载已经完成，人工筛选 可上传的占比约一半
# print('2 图片识别-命名')
# Tools.imgclassifier_contentimg.run()

# print('3 水印识别及处理')
# Tools.maskchecker.identifyMask_imgs_dir(imgs_dirpath=config['contentimg_dirpath_reconized'], templates_dirpath=config['template_dirpath'], dirpath_hasmask=config['contentimg_dirpath_hasmask'], dirpath_donthasmask=config['contentimg_dirpath_donthasmask'])

# step1
# print('4 resize内容图 图片 默认为宽度小1000')
# imgSetSizer = processing.ImgsSetSize(imgsOriDirPath=config['contentimg_dirpath_hasmask'])
# imgSetSizer.resize_byDir()

# print('5 上传无水印的内容图')
# poster = poster_img.Poster_Imgs(imgDirPath=config['contentimg_dirpath_hasmask'])
# poster.updateImgs()

# step2
# print('4 resize内容图 图片 默认为宽度小1000')
# imgSetSizer = processing.ImgsSetSize(imgsOriDirPath=config['contentimg_dirpath_donthasmask'])
# imgSetSizer.resize_byDir()

# print('5 上传无水印的内容图')
# poster = poster_img.Poster_Imgs(imgDirPath=config['contentimg_dirpath_donthasmask'])
# poster.updateImgs()



from . import base_cleaner
from basement__ import ContralerDatabase

'''
    针对段落的清洁器类
'''
class Cleaner_Paragraph(base_cleaner.Base_Cleaner):
    def __init__(self):
        self.cleanWordList = [
            '●', '但是，', '所以，', '再说，', '虽然说，', '另外，', '最后，', '而且，',
            '其次，', '首先，', '再者，', '同时，', '不过，', '当然，', '当然啦，', '那么，', '虽然，',
            '其实，', '通常，', '接着，', '综上所述，', '因此，', '、'
        ]
        self.irrelevantWordList = ['图']
        self.baseClass = base_cleaner.Base_Cleaner  # 父类
        for i in range(1, 10):
            self.cleanWordList.append(str(i) + '.')
            self.cleanWordList.append(str(i) + '。')
            self.cleanWordList.append(str(i) + '，')
            self.cleanWordList.append(str(i) + '、')
            self.cleanWordList.append(str(i) + '：')
            self.cleanWordList.append('（' + str(i) + '）')
            self.cleanWordList.append(str(i) + '）')
            self.cleanWordList.append('(' + str(i) + ')')
            self.cleanWordList.append('[' + str(i) + ']')
            self.cleanWordList.append('【' + str(i) + '】')
        for item in self.universal_cleanWordDict['numList1']:
            self.cleanWordList.append(item + '、')
            self.cleanWordList.append(item + '。')
            self.cleanWordList.append('(' + item + ')')
        for item in self.universal_cleanWordDict['numList2']:
            self.cleanWordList.append(item + '、')
            self.cleanWordList.append(item + '，')
        for item in self.universal_cleanWordDict['alphabeticList']:
            self.cleanWordList.append(item + '.')

        self.cleanWordList = tuple(self.cleanWordList)

        # 从数据库获取股票名字和代码
        sql = "select name, code from `tb_namecode`"
        dbOperator = ContralerDatabase.Contraler_Database('stocksnamecode')
        self.stocksNameCodeList = dbOperator.getAllDataFromDB(sql)
        dbOperator.closeDb()
        del dbOperator

    def integratedOp(self, paragraph):
        # 清序号
        temp = self.baseClass.delHeaderNum(paragraph, cleanWordList=self.cleanWordList)
        # 清空格
        temp = self.baseClass.delSpace(temp)
        # 清序号
        temp = self.baseClass.delHeaderNum(temp, cleanWordList=self.cleanWordList)
        # 清股票
        # temp = self.delStocksNameAndCode(temp)
        # 清不相关的文字
        result = self.baseClass.delIrrelevantWord(temp, irrelevantWordList=self.irrelevantWordList)
        return result




import os, time, requests, hashlib
from requests_toolbelt import MultipartEncoder
from basement__ import Tools as tools
from basement__ import ContralerDatabase as dbOp
from ..Filter import filter_video
from io import TextIOWrapper, BytesIO
from . import base_poster
from customFunction__.imgref import classifier
'''
    Post 视频的类
        参数：
            videoDirPath  处理完的图片目录路径dir   注意这里的路径最后加\\的

'''

class Poster_Video(base_poster.Base_Poster):
    # 传入的dirPath必须是这样的格式： XX:\\XX\\ 最后是有 \\ 的
    def __init__(self, videoDirPath, coverSavedPath, interface='http://121.40.187.51:8088/api/videos_api'):
        self.videoDirPath = videoDirPath
        self.get_videoPathList()
        self.interface = interface
        self.userName = 'qin'
        self.password = 'qin123456'
        self.curDate = str(tools.getCurDate())
        self.key = hashlib.md5(('datapool' + self.userName + self.password + self.curDate).encode('utf-8')).hexdigest()
        # self.videoPathFilteredList = videoPathFilteredList
        self.coverSavedPath = coverSavedPath


    # 获取目录下所有文件的路径列表
    def get_videoPathList(self):
        videoNameList = os.listdir(self.videoDirPath)  # 获取目录下的所有文件名
        videoPathList = []
        for videoName in videoNameList:
            item = self.videoDirPath + '\\' + videoName
            videoPathList.append(item)
        self.videoNameList = videoNameList
        return videoNameList

    # 获取时间戳
    def getCurTime(self):
        return int(round(time.time()))

    def changeMD5(self, videoSrc):
        with open(videoSrc, 'rb') as f:
            md5 = hashlib.md5(f.read()).hexdigest()
        file = open(videoSrc, 'rb').read()
        with open(videoSrc, 'wb') as new_file:
            new_file.write(file + bytes('\0', encoding='utf-8'))  # here we are adding a null to change the file content
            newMD5 = hashlib.md5(open(videoSrc, 'rb').read()).hexdigest()

    # ndarray图像转换io TextIOWrapper
    def translateFrom_Ndarray2TIWrapper(self, ndFile):
        str_encode = ndFile.tostring()
        # str_encode = np.int64(np.all(ndFile[:, :, :3] == 0, axis=2))
        f4 = BytesIO(str_encode)
        f5 = TextIOWrapper(f4)
        return f5

    # 从数据库获取名字
    def getTitleFromDB(self, titleId):
        dbOperator = dbOp("articledatabase")
        sql = "SELECT videoTitle FROM `tb_videodouyingangang` WHERE `id`={};".format(
            titleId
        )
        title = dbOperator.getOneDataFromDB(sql)[0]
        return title

    def post_videoSingle(self, videoName, title0):
        coverSavedPath = self.coverSavedPath
        videoPath = self.videoDirPath + videoName
        print("处理的路径： ", videoPath)
        imgfil = filter_video.Filter_Video(dirOriPath=self.videoDirPath)
        # imgf = self.translateFrom_Ndarray2TIWrapper(imgfil.getCoverImg(videoPath, coverSavedPath))  # 该获取截图的方法无效——无法上传
        # 判断是否满足条件
        if (imgfil.checkIfTimeLength(videoPath)):
            imgfil.getCoverImg(videoPath, coverSavedPath)  # 截图并保存

            # 识别截图看是否是走线图 是则不上传
            # res = classifier.ImgsClassifier.reconizeImg(coverSavedPath)
            # if(len(res)!=2 or '走势图' not in res and '屏幕截图' not in res):
            #     return 0

            imgf = open(coverSavedPath, 'rb')

            videof = open(videoPath, 'rb')

            # 这里传文件的时候用绝对路径传，不然传了之后显示不了
            formData = ({
                "key": self.key,
                "account": self.userName,
                "password": self.password,
                'title': tools.cleanTitle(title0),
                'img': ('cover.jpg', imgf, "image/jpeg"),
                'video': (videoName, videof)
            })
            m = MultipartEncoder(formData)
            headers2 = {
                "Content-Type": m.content_type
            }
            videoPostResult = requests.post(url=self.interface, data=m, headers=headers2)
            print("上传完成")
            imgf.close()
            videof.close()
            return videoPostResult

    # 发送目录下的所有所有视频
    def updateVideos(self):
        # for videoName in self.videoNameList:
        #     self.post_videoSingle(videoName)
        for videoName in self.videoNameList:
            self.post_videoSingle(videoName)




'''
    自动化引擎
        视频 筛选 并上传
'''
from .globalTools import globalTools
from .videoRef.universalTools import tools
from .videoRef.Spider import bySelenium
from .videoRef.DatabaserOperator import databaseOperator as dbOp
from .videoRef.Poster.Poster import VideoPoster
from .videoRef.Filter.Filter import videoFilter
from fake_useragent import UserAgent

def check_hasvideoindb0(checkDate_time, titlePostedList_,item):
    # 过滤数据库中是否存在对应视频方案1
    if(int(item[3]) > checkDate_time and item[0] not in titlePostedList_):
        return True
    else:
        return False

def check_hasvideoindb1(posted_dbOp, checkDate_time,item):
    sql = "SELECT * FROM `postedurldatabase`.`tb_video_posted` WHERE `title`='{}';".format(item[0])
    res = posted_dbOp.getOneDataFromDB(sql)
    # 过滤数据库中是否存在对应视频方案1
    if(int(item[3]) > checkDate_time and not res):
        return True
    else:
        return False

def check_videoRepeat(posted_dbOp, videoInfo):
    sql = "SELECT * FROM `postedurldatabase`.`` WHERE `title`='{}';".format(videoInfo[0])
    res = posted_dbOp.getAllDataFromDB()
    if(res):
        # 重复 返回True
        return True
    else:
        return False

# 爬取源1 bilibili
def run_bilibili(setting):
    # setting['videoDirPath'] = 'E:\\test\\'
    dbOperator = dbOp.dbOperator('videodatabase')    # 获取未上传的数据
    posted_dbOp = dbOp.dbOperator(databaseName='postedurldatabase')     # 连接上传过的的数据的数据库
    poster = VideoPoster(videoDirPath=setting['videoDirPath'], coverSavedPath=setting['coverSavedPath'])
    filter_video = videoFilter()
    # 获取最新爬取下来待上传的视频信息列表
    sql = "SELECT title, avValue, videoUrl, pubdate FROM `videodatabase`.`tb_bilibili_videoInfo`;"
    videoInfoList = dbOperator.getAllDataFromDB(sql)    # 未上传的数据

    i = 1

    # 获取上传过的视频title列表
    titlePostedList = posted_dbOp.getAllDataFromDB("SELECT `title` FROM `postedurldatabase`.`tb_video_posted`;")
    titlePostedList_ = []
    for one in titlePostedList:
        titlePostedList_.append(one[0])

    checkDate_time = int(dbOperator.getOneDataFromDB("SELECT * FROM `videodatabase`.`tb_bilibili_posted_timethenewest`;")[1])

    # 过滤标题操作
    videoInfoList = filter_video.filter_keywordFromTitle(videoInfoList)
    # 针对哔哩哔哩的过滤
    videoInfoList = filter_video.filter_keywordFromTitle4bilibili(videoInfoList)
    if(videoInfoList):
        newestPubdate = checkDate_time  # 上传成功的最近的一次pubdate`bilibilidatabase`.`tb_posted_timethenewest` SET `timethenewest`
        for item in videoInfoList:
            if(check_hasvideoindb1(posted_dbOp, checkDate_time, item)):

                refererUrl = 'https://www.bilibili.com/video/av' + str(item[1])
                vid_headers = {
                    'Origin': 'https://www.bilibili.com',
                    'Referer': refererUrl,
                    'User-Agent':  str(UserAgent().random)
                }
                try:
                    # 视频数据发布时间在当天
                    tools.downVideo(urlpath=item[2], name=str(i), dstDirPath=setting['videoDirPath'], headers_=vid_headers)
                    checkIfSuccess = True
                except Exception as e:
                    checkIfSuccess = False
                    print("视频下载出错: ", item)

                if(not checkIfSuccess):
                    # 视频下载出错都不用上传了直接跳过
                    continue

                # 上传
                print("上传视频: ", i)
                try:
                    postResult = poster.post_videoSingle(str(i) + '.mp4', title0=item[0])
                    newestPubdate = int(item[3])   # 对应上传视频的时间戳
                except Exception as e:
                    checkIfSuccess = False
                    print("视频上传出错: ", postResult)
                finally:
                    try:
                        # 上传完删除对应单个视频
                        if(i!=1):
                            globalTools.delVideoSingle(setting['videoDirPath'] + str(i-1) + '.mp4')
                    except Exception as e:
                        print("删除上个视频出错： ", str(i-1) + '.mp4')
                        print(e)

                if(checkIfSuccess):
                    # 更新上传过的数据库 postedurldatabase
                    sql = "INSERT INTO `postedurldatabase`.`tb_video_posted` (`title`) VALUES ('{}');".format(
                        item[0]
                    )
                    posted_dbOp.insertData2DB(sql=sql)
                i = i+1
            else:
                print("不符合条件，无法上传")
                continue
        # 上传一切顺利，更新最新上传的视频的时间戳
        sql_update = "UPDATE `videodatabase`.`tb_bilibili_posted_timethenewest` SET `timethenewest`=\'{}\' WHERE (`id` = '1');".format(
            newestPubdate
        )
        dbOperator.insertData2DB(sql_update)
    else:
        print("数据库为空，无待上传的数据")
    globalTools.finishTask()






'''
    自动化引擎
        从关键段落数据库中获取数据 进行 清洗 筛选 并上传 （已完成）
'''
from .globalTools import globalTools
from .articlesRef.DatabaserOperator import databaseOperator as dbOp
from .articlesRef.Filter import Filter
from .articlesRef.Poster import Poster

def run(setting):
    # setting = {
    #     'databaseName': 'magicsunshadingdatabase',
    #     'databaseUser': 'root',
    #     'databasePasswd': 'root',
    #     'tableName': 'tb_articlecontent',
    #     'tableName4Tag': 'tb_articleinfo',
    #     'tagRefSqlKind': 'url',
    #     'domain' : 'magicsunshading'
    # }

    if(setting['domain'] == 'peizitoutiao'):
        for topicName in setting['topicNameList']:
            tableName = 'tb_' + topicName + '_articlecontent'
            tableName4Tag = 'tb_' + topicName + '_articleinfo'
            setting['tableName'] = tableName
            setting['tableName4Tag'] = tableName4Tag
            setting['sql'] = "SELECT * FROM " + setting['databaseName'] + "."+ setting['tableName'] +";"
            # 1 获取对应数据
            dbOperator = dbOp.dbOperator(databaseName=setting['databaseName'], user=setting['databaseUser'], passwd=setting['databasePasswd'])
            dataList = dbOperator.getAllDataFromDB(setting['sql'])

            # 2 对所有段落内容判断，若上传过则删除对应上传过的段落
            filter4postedcheck = Filter.Filter_Posted()
            dataList = filter4postedcheck.run(dataList=dataList)

            # 3 过滤操作 输入的列表为从数据库中获取的列表（过滤操作包含清洗操作，不用单独进行清洗）
            filterInstance = Filter.Filter()
            postableList = filterInstance.integratedOp4List_keyParagraph(
                paragraphList=dataList,
                databaseName=setting['databaseName'],
                tableName=setting['tableName'],
                tableName4Tag=setting["tableName4Tag"],
                tagRefSqlKind='url'
            )

            # 5 将上传过的数据放到postedurldatabase中
            postedDBOP = dbOp.dbOperator(databaseName='postedurldatabase', user='root', passwd='root')
            for paragraph in postableList:
                sql = "INSERT INTO `postedurldatabase`.`tb_article_posted` (`paragraph`) VALUES (\'{}\');".format(
                    paragraph[0])
                postedDBOP.insertData2DB(sql)
        globalTools.finishTask()


    elif(setting['domain'] == 'magicsunshading'):
        print("here is magicsunshading")
        setting['sql'] = "SELECT * FROM " + setting['databaseName'] + "."+ setting['tableName'] +";"

        # 1 获取对应数据
        dbOperator = dbOp.dbOperator(databaseName=setting['databaseName'], user=setting['databaseUser'], passwd=setting['databasePasswd'])
        dataList = dbOperator.getAllDataFromDB(setting['sql'])

        # 2 对所有段落内容判断，若上传过则删除对应上传过的段落
        filter4postedcheck = Filter.Filter_Posted()
        dataList = filter4postedcheck.run(dataOriList=dataList)

        # 3 过滤操作 输入的列表为从数据库中获取的列表（过滤操作包含清洗操作，不用单独进行清洗）
        filterInstance = Filter.Filter()
        postableList = filterInstance.integratedOp4List_keyParagraph(
            paragraphList=dataList,
            databaseName=setting['databaseName'],
            tableName=setting['tableName'],
            tableName4Tag=setting["tableName4Tag"],
            tagRefSqlKind='url'
        )

        print("过滤操作完成， 接下来完成上传操作")
        # 4 上传列表
        posterInstance = Poster.Poster(interface='http://121.40.187.51:8088/api/key_paragraph_api')
        posterInstance.post_auto(postableList, whichKind='keyParagraph')

        print("上传操作完成， 接下来完成 postedurldatabase 数据库的更新操作")
        # 5 将上传过的数据放到postedurldatabase中
        postedDBOP = dbOp.dbOperator(databaseName='postedurldatabase', user='root',passwd='root')
        for paragraph in postableList:
            sql = "INSERT INTO `postedurldatabase`.`tb_article_posted` (`paragraph`) VALUES (\'{}\');".format(paragraph[0])
            postedDBOP.insertData2DB(sql)
        globalTools.finishTask()
